@@ -34,6 +34,7 @@ extern "C" {
 }
 
 #include <pan/general/debug.h>
+#include <pan/general/file-util.h>
 #include <pan/general/foreach.h>
 #include <pan/general/log.h>
 #include <pan/general/messages.h>
@@ -162,11 +163,14 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
   s.clear ();
   u.clear ();
 
-  const quarks_t servers (data_io.get_newsrc_servers ());
-  foreach_const (quarks_t, servers, sit) {
-    LineReader * in (data_io.read_newsrc (*sit));
-    load_newsrc (*sit, in, s, u);
-    delete in;
+  foreach_const (servers_t, _servers, sit) {
+    const Quark key (sit->first);
+    const std::string filename (sit->second.newsrc_filename);
+    if (file::file_exists (filename.c_str())) {
+      LineReader * in (data_io.read_file (filename));
+      load_newsrc (key, in, s, u);
+      delete in;
+    }
   }
 
   // remove duplicates
@@ -189,6 +193,9 @@ DataImpl :: load_newsrc_files (const DataIO& data_io)
 void
 DataImpl :: save_newsrc_files (DataIO& data_io) const
 {
+  if (_unit_test)
+    return;
+
   // overly-complex optimization: both sit->second.groups and _subscribed
   // are both ordered by AlphabeticalQuarkOrdering.  
   // Where N==sit->second.groups.size() and M==_subscribed.size(),
@@ -202,7 +209,8 @@ DataImpl :: save_newsrc_files (DataIO& data_io) const
     const Quark& server (sit->first);
 
     // write this server's newsrc
-    std::ostream * out (data_io.write_newsrc (server));
+    const std::string& filename (sit->second.newsrc_filename);
+    std::ostream * out (data_io.write_file (filename));
     std::string newsrc_string;
     alpha_groups_t::const_iterator sub_it (_subscribed.begin());
     const alpha_groups_t::const_iterator sub_end(_subscribed.end());
@@ -274,6 +282,9 @@ DataImpl :: load_group_permissions (const DataIO& data_io)
 void
 DataImpl :: save_group_permissions (DataIO& data_io) const
 {
+  if (_unit_test)
+    return;
+
   std::ostream * out (data_io.write_group_permissions ());
 
   typedef std::map<Quark, char, AlphabeticalQuarkOrdering> tmp_t;
@@ -352,6 +363,9 @@ DataImpl :: load_group_xovers (const DataIO& data_io)
 void
 DataImpl :: save_group_descriptions (DataIO& data_io) const
 {
+  if (_unit_test)
+    return;
+
   assert (_descriptions_loaded);
 
   typedef std::map<Quark, std::string, AlphabeticalQuarkOrdering> tmp_t;
@@ -383,6 +397,9 @@ namespace
 void
 DataImpl :: save_group_xovers (DataIO& data_io) const
 {
+  if (_unit_test)
+    return;
+
   std::ostream * out (data_io.write_group_xovers ());
 
   // find the set of groups that have had an xover
@@ -538,6 +555,7 @@ DataImpl :: mark_group_read (const Quark& groupname)
       it->second._read.mark_range (0, it->second._xover_high, true);
     }
     rg->_unread_count = 0;
+    save_group_xovers (*_data_io);
     fire_group_read (groupname);
   }
 }
