@@ -18,13 +18,15 @@
  */
 
 #include <config.h>
-#include <errno.h>
+#include <cerrno>
 #include <fstream>
 #include <map>
 #include <vector>
 #include <string>
 #include <cmath>
 extern "C" {
+  #include <sys/types.h> // for chmod
+  #include <sys/stat.h> // for chmod
   #include <glib/gi18n.h>
   #include <glib/gmessages.h> // for g_assert
 }
@@ -34,8 +36,8 @@ extern "C" {
 #include <pan/general/messages.h>
 #include <pan/general/quark.h>
 #include <pan/general/time-elapsed.h>
+#include <pan/usenet-utils/filter-info.h>
 #include <pan/data/article.h>
-#include <pan/data/filter-info.h>
 #include "article-filter.h"
 #include "data-impl.h"
 
@@ -136,10 +138,9 @@ DataImpl :: GroupHeaders :: build_references_header (const Article* article, std
   setme.clear ();
   const Quark& message_id (article->message_id);
   const ArticleNode * node (find_node (message_id));
-  StringView ancestor_mid;
-  while (node->_parent != 0) {
+  while (node && node->_parent) {
     node = node->_parent;
-    node->_mid.to_view (ancestor_mid);
+    const StringView& ancestor_mid = node->_mid.to_view ();
     setme.insert (0, ancestor_mid.str, ancestor_mid.len);
     if (node->_parent)
       setme.insert (0, 1, ' ');
@@ -189,9 +190,8 @@ DataImpl :: unref_group   (const Quark& group)
   //std::cerr << LINE_ID << " group " << group << " refcount down to " << h->_ref << std::endl;
   if (h->_ref == 0)
   {
-    if (h->_dirty) {
+    if (h->_dirty)
       save_headers (*_data_io, group);
-    }
     h->_dirty = false;
     free_group_headers_memory (group);
   }
@@ -521,7 +521,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
         }
         targets_v.resize (target_it - &targets_v.front());
         targets.sort();
-        const bool expired (targets.empty());
+        bool expired (targets.empty());
         a.xref.swap (targets);
 
         // is_binary [total_part_count found_part_count]
@@ -549,6 +549,10 @@ DataImpl :: load_headers (const DataIO   & data_io,
             s.ltrim ();
             s.pop_token (tok);
             const unsigned long number (view_to_ul (tok));
+            if (!a.has_part(number)) { // corrupted entry
+              expired = true;
+              break;
+            }
             Article::Part& p (a.get_part (number));
             s.ltrim ();
             s.pop_token (tok);
@@ -601,65 +605,59 @@ DataImpl :: load_headers (const DataIO   & data_io,
 
 namespace
 {
-  typedef std::map < pan::Quark, char > quark_to_symbol_t;
-
-  struct QuarkToSymbol {
-    ~QuarkToSymbol () {}
-    QuarkToSymbol (const quark_to_symbol_t& map): _map(map) { }
-    const quark_to_symbol_t _map;
-    const char* operator() (const Quark& quark) const {
-      static char buf[2];
-      quark_to_symbol_t::const_iterator it (_map.find (quark));
-      if (it == _map.end())
-        return quark.c_str();
-      buf[0] = it->second;
-      buf[1] = '\0';
-      return buf;
-    }
-  };
-
-  typedef std::map<Quark,unsigned long> frequency_t;
-
   const char * lookup_symbols ("abcdefghijklmnopqrstuvwxyz"
                                "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                                "1234567890!@#$%^&*()");
 
-  QuarkToSymbol build_qts (frequency_t& freq)
+  struct QuarkToSymbol
   {
-    quark_to_symbol_t qts_map;
-    for (int i(0), len(strlen(lookup_symbols)); i<len && !freq.empty(); ++i)
+    typedef Loki::AssocVector < pan::Quark, char > quark_to_symbol_t;
+    quark_to_symbol_t _map;
+
+    const char* operator() (const Quark& quark) const
     {
-      // find the most frequently-used entry still in freq
-      unsigned long max (0);
-      Quark q;
-      foreach_const (frequency_t, freq, it) {
-        if (it->second > max) {
-          max = it->second;
-          q = it->first;
-        }
+      static char buf[2];
+      quark_to_symbol_t::const_iterator it (_map.find (quark));
+      if (it == _map.end())
+        return quark.c_str();
+
+      buf[0] = it->second;
+      buf[1] = '\0';
+      return buf;
+    }
+
+    void write (std::ostream& out, const StringView& comment) const
+    {
+      Quark quarks[UCHAR_MAX];
+      foreach_const (quark_to_symbol_t, _map, it)
+        quarks[(int)it->second] = it->first;
+
+      const size_t len (_map.size());
+      out << len;
+      if (!comment.empty())
+        out << "\t # " << comment << '\n';
+      for (size_t i(0); i!=len; ++i) {
+        const char ch (lookup_symbols[i]);
+        out << '\t' << ch << '\t' << quarks[(int)ch] << '\n';
       }
-      //std::cerr << "[" << q << "] occurred " << max << " times" << std::endl;
-      qts_map[q] = lookup_symbols[i];
-      freq.erase (q);
     }
-    freq.clear ();
-    return QuarkToSymbol (qts_map);
-  }
+  };
 
-  void write_qts (std::ostream* out, const QuarkToSymbol& qts, const StringView& comment)
+  typedef Loki::AssocVector<Quark,unsigned long> frequency_t;
+
+  void build_qts (frequency_t& freq, QuarkToSymbol& setme)
   {
-    Quark quarks[UCHAR_MAX];
-    foreach_const (quark_to_symbol_t, qts._map, it)
-      quarks[(int)it->second] = it->first;
+    setme._map.clear ();
 
-    const size_t len (qts._map.size());
-    *out << len;
-    if (!comment.empty())
-      *out << "\t # " << comment << '\n';
-    for (size_t i(0); i!=len; ++i) {
-      const char ch (lookup_symbols[i]);
-      *out << '\t' << ch << '\t' << quarks[(int)ch] << '\n';
-    }
+    typedef std::multimap<unsigned long,Quark> counts_t;
+    counts_t counts;
+    foreach_const (frequency_t, freq, it)
+      counts.insert (std::pair<unsigned long,Quark>(it->second,it->first));
+
+    counts_t::const_reverse_iterator it=counts.rbegin(), end=counts.rend();
+    for (const char * pch=lookup_symbols; *pch && it!=end; ++pch, ++it)
+      setme._map[it->second] = *pch;
+    freq.clear ();
   }
 }
 
@@ -720,24 +718,26 @@ DataImpl :: save_headers (DataIO                       & data_io,
     *out << "2\t # file format version number\n";
 
     // xref lookup section
-    typedef std::map<Quark,unsigned long> frequency_t;
     frequency_t frequency;
     foreach_const (std::vector<Article*>, articles, ait)
       foreach_const (Xref, (*ait)->xref, xit)
         ++frequency[xit->group];
-    const QuarkToSymbol xref_qts (build_qts (frequency));
-    write_qts (out, xref_qts, "xref shorthand count");
+    QuarkToSymbol xref_qts;
+    build_qts (frequency, xref_qts);
+    xref_qts.write (*out, "xref shorthand count");
 
     // author lookup section
     frequency.clear ();
     foreach_const (std::vector<Article*>, articles, ait)
       ++frequency[(*ait)->author];
-    const QuarkToSymbol author_qts (build_qts (frequency));
-    write_qts (out, author_qts, "author shorthand count");
+    QuarkToSymbol author_qts;
+    build_qts (frequency, author_qts);
+    author_qts.write (*out, "author shorthand count");
 
     // header section
     *out << articles.size() << endl;
     std::string references;
+    std::string part_mid;
     foreach_const (std::vector<Article*>, articles, ait)
     {
       ++article_count;
@@ -782,11 +782,11 @@ DataImpl :: save_headers (DataIO                       & data_io,
           ++part_count;
           const Article::Part& p (*pit);
           *out << '\t' << number << ' ';
-          const std::string tmp (p.get_message_id (message_id));
-          if (message_id.to_view() == tmp)
+          p.get_message_id (message_id, part_mid);
+          if (message_id.to_view() == part_mid)
             *out << '"';
           else
-            *out << tmp;
+            *out << part_mid;
           *out << ' ' << p.bytes << '\n';
            first = false;
         }
@@ -795,6 +795,7 @@ DataImpl :: save_headers (DataIO                       & data_io,
 
     success = !out->fail ();
     data_io.write_done (out);
+    save_group_xovers (data_io);
   }
 
   return success;
@@ -803,6 +804,9 @@ DataImpl :: save_headers (DataIO                       & data_io,
 void
 DataImpl :: save_headers (DataIO& data_io, const Quark& group) const
 {
+  if (_unit_test)
+    return;
+
    pan_return_if_fail (!group.empty());
 
    TimeElapsed timer;
@@ -847,14 +851,12 @@ DataImpl :: mark_read (const Article  ** articles,
                        unsigned long     article_count,
                        bool              read)
 {
-  quarks_t mids;
   typedef std::map<Quark,quarks_t> group_to_changed_mids_t;
   group_to_changed_mids_t group_to_changed_mids;
 
   // set them to `read'...
   for (const Article **it(articles), **end(articles+article_count); it!=end; ++it) {
     const Article * article (*it);
-    mids.insert (article->message_id);
     foreach_const (Xref, article->xref, xit) {
       const bool old_state (_read_groups[xit->group][xit->server]._read.mark_one (xit->number, read));
       if (!old_state != !read)
@@ -872,10 +874,8 @@ DataImpl :: mark_read (const Article  ** articles,
     else
       g._unread_count += n;
     fire_group_counts (group, g._unread_count, g._article_count);
+    on_articles_changed (group, it->second, false);
   }
-
-  // fire an articles changed event...
-  on_articles_changed (mids, false);
 }
 
 
@@ -904,6 +904,25 @@ DataImpl :: get_article_scores (const Quark         & group,
   _article_filter.get_article_scores (*this, sections, group, article, setme);
 }
 
+void
+DataImpl :: rescore_articles (const Quark& group, const quarks_t mids)
+{
+  GroupHeaders * gh (get_group_headers (group));
+  if (!gh) // group isn't loaded
+    return;
+
+  ArticleFilter::sections_t sections;
+  _scorefile.get_matching_sections (group.to_view(), sections);
+  nodes_v nodes;
+  find_nodes (mids, gh->_nodes, nodes);
+  foreach (nodes_v, nodes, it) {
+    if ((*it)->_article) {
+      Article& a (*(*it)->_article);
+      a.score = _article_filter.score_article (*this, sections, group, a);
+    }
+  }
+}
+
 void
 DataImpl :: rescore ()
 {
@@ -919,35 +938,16 @@ DataImpl :: rescore ()
   foreach (std::set<MyTree*>, _trees, it)
     groups.insert ((*it)->_group);
 
-  // rescore each group...
-  quarks_t changed_mids;
-  foreach_const (quarks_t, groups, git)
-  {
+  // "on_articles_changed" rescores the articles...
+  foreach_const (quarks_t, groups, git) {
+    quarks_t mids;
     const Quark& group (*git);
-    //std::cerr << LINE_ID << " rescoring " << group << std::endl;
-
-    ArticleFilter::sections_t score_sections;
-    _scorefile.get_matching_sections (StringView(group), score_sections);
-
     const GroupHeaders * h (get_group_headers (group));
-    foreach_const (nodes_t, h->_nodes, nit) {
-      Article * article (nit->second->_article);
-      if (article) {
-        const int old_score (article->score);
-        article->score = _article_filter.score_article (
-                                      *this, score_sections, group, *article);
-        if (old_score != article->score)
-          changed_mids.insert (article->message_id);
-      }
-    }
+    foreach_const (nodes_t, h->_nodes, nit)
+      mids.insert (mids.end(), nit->first);
+    if (!mids.empty())
+      on_articles_changed (group, mids, true);
   }
-
-  //std::cerr << LINE_ID << ' ' << changed_mids.size()
-  //                     << " articles changed score\n";
-
-  // event notification...
-  if (!changed_mids.empty())
-    on_articles_changed (changed_mids, true);
 }
 
 void
@@ -971,6 +971,7 @@ DataImpl :: add_score (const StringView           & section_wildmat,
     std::ofstream o (filename.c_str(), std::ofstream::app|std::ofstream::out);
     o << '\n' << str << '\n';
     o.close ();
+    ::chmod (filename.c_str(), 0600);
   }
 
   if (do_rescore)
@@ -999,9 +1000,11 @@ DataImpl :: comment_out_scorefile_line (const StringView    & filename,
   in.close ();
 
   // ..and back out again
-  std::ofstream o (filename.to_string().c_str(), std::ofstream::trunc|std::ofstream::out);
+  const std::string f (filename.str, filename.len);
+  std::ofstream o (f.c_str(), std::ofstream::trunc|std::ofstream::out);
   o << buf;
   o.close ();
+  ::chmod (f.c_str(), 0600);
 
   // rescore
   if (do_rescore)
@@ -1104,8 +1107,11 @@ DataImpl :: on_articles_removed (const quarks_t& mids) const
 }
 
 void
-DataImpl :: on_articles_changed (const quarks_t& mids, bool do_refilter) const
+DataImpl :: on_articles_changed (const Quark& group, const quarks_t& mids, bool do_refilter)
 {
+  rescore_articles (group, mids);
+
+  // notify the trees that the articles have changed...
   foreach (std::set<MyTree*>, _trees, it)
     (*it)->articles_changed (mids, do_refilter);
 }
@@ -1118,6 +1124,8 @@ DataImpl :: on_articles_added (const Quark& group, const quarks_t& mids)
     Log::add_info_va (_("Added %lu articles to %s."),
                       mids.size(), group.c_str());
 
+    rescore_articles (group, mids);
+
     foreach (std::set<MyTree*>, _trees, it) {
       debug ("This tree has a group " << (*it)->_group);
       if ((*it)->_group == group) {
