@@ -50,8 +50,6 @@ namespace
   {
     { icon_article_read, "ICON_ARTICLE_READ" },
     { icon_article_unread, "ICON_ARTICLE_UNREAD" },
-    { icon_bluecheck, "ICON_BLUECHECK" }, 
-    { icon_blue_flag, "ICON_BLUE_FLAG" }, 
     { icon_compose_followup, "ICON_COMPOSE_FOLLOWUP" }, 
     { icon_compose_post, "ICON_COMPOSE_POST" }, 
     { icon_disk, "ICON_DISK" }, 
@@ -63,7 +61,6 @@ namespace
     { icon_get_dialog, "ICON_GET_DIALOG" }, 
     { icon_get_selected, "ICON_GET_SELECTED" }, 
     { icon_get_subscribed, "ICON_GET_SUBSCRIBED" }, 
-    { icon_pan, "ICON_PAN" }, 
     { icon_read_group, "ICON_READ_GROUP" }, 
     { icon_read_more, "ICON_READ_MORE" }, 
     { icon_read_less, "ICON_READ_LESS" }, 
@@ -101,6 +98,7 @@ namespace
 
   GtkActionGroup * _group (0);
 
+  void do_prompt_for_charset           (GtkAction * a) { pan_ui->do_prompt_for_charset(); }
   void do_read_selected_group          (GtkAction * a) { pan_ui->do_read_selected_group(); }
   void do_mark_selected_groups_read    (GtkAction * a) { pan_ui->do_mark_selected_groups_read(); }
   void do_clear_selected_groups        (GtkAction * a) { pan_ui->do_clear_selected_groups(); }
@@ -134,6 +132,7 @@ namespace
   void do_rot13_selected_text          (GtkAction * a) { pan_ui->do_rot13_selected_text(); }
   void do_download_selected_article    (GtkAction * a) { pan_ui->do_download_selected_article(); }
   void do_read_selected_article        (GtkAction * a) { pan_ui->do_read_selected_article(); }
+  void do_show_selected_article_info   (GtkAction * a) { pan_ui->do_show_selected_article_info(); }
   void do_read_more                    (GtkAction * a) { pan_ui->do_read_more(); }
   void do_read_less                    (GtkAction * a) { pan_ui->do_read_less(); }
   void do_read_next_unread_group       (GtkAction * a) { pan_ui->do_read_next_unread_group(); }
@@ -155,6 +154,7 @@ namespace
   void do_cancel_article               (GtkAction * a) { pan_ui->do_cancel_article(); }
   void do_supersede_article            (GtkAction * a) { pan_ui->do_supersede_article(); }
   void do_delete_article               (GtkAction * a) { pan_ui->do_delete_article(); }
+  void do_clear_article_cache          (GtkAction * a) { pan_ui->do_clear_article_cache(); }
   void do_mark_article_read            (GtkAction * a) { pan_ui->do_mark_article_read(); }
   void do_mark_article_unread          (GtkAction * a) { pan_ui->do_mark_article_unread(); }
   void do_post                         (GtkAction * a) { pan_ui->do_post(); }
@@ -258,7 +258,6 @@ namespace
     { "view-header-pane-menu", NULL, N_("_Header Pane") },
     { "view-body-pane-menu", NULL, N_("_Body Pane") },
     { "view-menu", NULL, N_("_View") },
-    { "character-encoding-menu", NULL, N_("Character _Encoding") },
     { "filter-menu", NULL, N_("Filte_r") },
     { "go-menu", NULL, N_("_Go") },
     { "actions-menu", NULL, N_("_Actions") },
@@ -267,20 +266,21 @@ namespace
     { "posting-actions-menu", NULL, N_("_Post") },
     { "post-menu", NULL, N_("_Post") },
     { "help-menu", NULL, N_("_Help") },
-    { "read-selected-group", "ICON_READ_MORE", N_("_Read Group"), NULL, N_("Read Selected Group"), G_CALLBACK(do_read_selected_group) },
-    { "mark-groups-read", GTK_STOCK_CLEAR, N_("_Mark Group _Read"), "<control><shift>M", N_("Mark Selected Group Read"), G_CALLBACK(do_mark_selected_groups_read) },
-    { "delete-groups-articles", GTK_STOCK_DELETE, N_("_Delete Group's Articles"), "<control><shift>Delete", N_("Delete Selected Group's Articles"), G_CALLBACK(do_clear_selected_groups) },
-    { "get-new-headers-in-selected-groups", "ICON_GET_SELECTED", N_("Get New _Headers in Selected Group"), "A", N_("Get New Headers for Selected Group"), G_CALLBACK(do_xover_selected_groups) },
-    { "get-new-headers-in-subscribed-groups", "ICON_GET_SUBSCRIBED", N_("Get New _Headers in Subscribed Groups"), "<shift>A", N_("Get New Headers for Subscribed Groups"), G_CALLBACK(do_xover_subscribed_groups) },
-    { "download-headers", "ICON_GET_DIALOG", N_("Get _Headers ..."), "", N_("Get Headers from Server"), G_CALLBACK(do_download_headers) },
-    { "refresh-group-list", NULL, N_("Refresh Group List"), "", N_("Get a new Group list from the Server"), G_CALLBACK(do_refresh_groups) },
-    { "subscribe", NULL, N_("_Subscribe"), NULL, N_("Subscribe to Selected Group"), G_CALLBACK(do_subscribe_selected_groups) },
-    { "unsubscribe", NULL, N_("_Unsubscribe"), NULL, N_("Unsubscribe from Selected Group"), G_CALLBACK(do_unsubscribe_selected_groups) },
+    { "set-charset", NULL, N_("Set Character _Encoding..."), NULL, NULL, G_CALLBACK(do_prompt_for_charset) },
+    { "read-selected-group", "ICON_READ_MORE", N_("_Read Group"), NULL, NULL, G_CALLBACK(do_read_selected_group) },
+    { "mark-groups-read", GTK_STOCK_CLEAR, N_("_Mark Group _Read"), "<control><shift>M", NULL, G_CALLBACK(do_mark_selected_groups_read) },
+    { "delete-groups-articles", GTK_STOCK_DELETE, N_("_Delete Group's Articles"), "<control><shift>Delete", NULL, G_CALLBACK(do_clear_selected_groups) },
+    { "get-new-headers-in-selected-groups", "ICON_GET_SELECTED", N_("Get New _Headers in Selected Group"), "A", NULL, G_CALLBACK(do_xover_selected_groups) },
+    { "get-new-headers-in-subscribed-groups", "ICON_GET_SUBSCRIBED", N_("Get New _Headers in Subscribed Groups"), "<shift>A", NULL, G_CALLBACK(do_xover_subscribed_groups) },
+    { "download-headers", "ICON_GET_DIALOG", N_("Get _Headers..."), "", NULL, G_CALLBACK(do_download_headers) },
+    { "refresh-group-list", NULL, N_("Refresh Group List"), "", NULL, G_CALLBACK(do_refresh_groups) },
+    { "subscribe", NULL, N_("_Subscribe"), NULL, NULL, G_CALLBACK(do_subscribe_selected_groups) },
+    { "unsubscribe", NULL, N_("_Unsubscribe"), NULL, NULL, G_CALLBACK(do_unsubscribe_selected_groups) },
 
-    { "save-articles", GTK_STOCK_SAVE, N_("_Save Articles ..."), "<shift>S", N_("Save Selected Articles ..."), G_CALLBACK(do_save_articles) },
+    { "save-articles", GTK_STOCK_SAVE, N_("_Save Articles..."), "<shift>S", NULL, G_CALLBACK(do_save_articles) },
     { "print", GTK_STOCK_PRINT, NULL, "<control>P", NULL, G_CALLBACK(do_print) },
-    { "import-tasks", GTK_STOCK_OPEN, N_("_Import NZB Files ..."), NULL, NULL, G_CALLBACK(do_import_tasks) },
-    { "cancel-last-task", GTK_STOCK_CANCEL, N_("_Cancel Last Task"), "<control>Delete", N_("Cancel Last Task"), G_CALLBACK(do_cancel_latest_task) },
+    { "import-tasks", GTK_STOCK_OPEN, N_("_Import NZB Files..."), NULL, NULL, G_CALLBACK(do_import_tasks) },
+    { "cancel-last-task", GTK_STOCK_CANCEL, N_("_Cancel Last Task"), "<control>Delete", NULL, G_CALLBACK(do_cancel_latest_task) },
     { "show-task-window", NULL, N_("_Task Manager"), NULL, NULL, G_CALLBACK(do_show_task_window) },
     { "show-log-dialog", GTK_STOCK_DIALOG_INFO, N_("_Event Log"), NULL, NULL, G_CALLBACK(do_show_log_window) },
     { "quit", NULL, N_("_Quit"), "<control>Q", NULL, G_CALLBACK(do_quit) },
@@ -301,42 +301,44 @@ namespace
     { "jump-to-body-tab", GTK_STOCK_JUMP_TO, N_("Jump to _Body Tab"), "3", NULL, G_CALLBACK(do_jump_to_body_tab) },
     { "rot13-selected-text", NULL, N_("_Rot13 Selected Text"), "<control><shift>R", NULL, G_CALLBACK(do_rot13_selected_text) },
 
-    { "clear-header-pane", NULL, N_("Clear _Header Pane"), NULL, N_("Clear Header Pane"), G_CALLBACK(do_clear_header_pane) },
-    { "clear-body-pane", NULL, N_("Clear _Body Pane"), NULL, N_("Clear Body Pane"), G_CALLBACK(do_clear_body_pane) },
+    { "clear-header-pane", NULL, N_("Clear _Header Pane"), NULL, NULL, G_CALLBACK(do_clear_header_pane) },
+    { "clear-body-pane", NULL, N_("Clear _Body Pane"), NULL, NULL, G_CALLBACK(do_clear_body_pane) },
 
-    { "download-selected-article", "ICON_DISK", N_("Cache Article"), NULL, N_("Cache Article"), G_CALLBACK(do_download_selected_article) },
-    { "read-selected-article", "ICON_READ_MORE", N_("Read Article"), NULL, N_("Read Article"), G_CALLBACK(do_read_selected_article) },
-    { "read-more", "ICON_READ_MORE", N_("Read _More"), "space", N_("Read More"), G_CALLBACK(do_read_more) },
-    { "read-less", "ICON_READ_LESS", N_("Read _Back"), "BackSpace", N_("Read Back"), G_CALLBACK(do_read_less) },
-    { "read-next-unread-group", "ICON_READ_UNREAD_GROUP", N_("Next _Unread Group"), "G", N_("Next Unread Group"), G_CALLBACK(do_read_next_unread_group) },
-    { "read-next-group", "ICON_READ_GROUP", N_("Next _Group"), "<shift>G", N_("Next Group"), G_CALLBACK(do_read_next_group) },
-    { "read-next-unread-article", "ICON_READ_UNREAD_ARTICLE", N_("Next _Unread Article"), "N", N_("Next Unread Article"), G_CALLBACK(do_read_next_unread_article) },
+    { "download-selected-article", "ICON_DISK", N_("Cache Article"), NULL, NULL, G_CALLBACK(do_download_selected_article) },
+    { "read-selected-article", "ICON_READ_MORE", N_("Read Article"), NULL, NULL, G_CALLBACK(do_read_selected_article) },
+    { "show-selected-article-info", NULL, N_("Show Article Information"), NULL, NULL, G_CALLBACK(do_show_selected_article_info) },
+    { "read-more", "ICON_READ_MORE", N_("Read _More"), "space", NULL, G_CALLBACK(do_read_more) },
+    { "read-less", "ICON_READ_LESS", N_("Read _Back"), "BackSpace", NULL, G_CALLBACK(do_read_less) },
+    { "read-next-unread-group", "ICON_READ_UNREAD_GROUP", N_("Next _Unread Group"), "G", NULL, G_CALLBACK(do_read_next_unread_group) },
+    { "read-next-group", "ICON_READ_GROUP", N_("Next _Group"), "<shift>G", NULL, G_CALLBACK(do_read_next_group) },
+    { "read-next-unread-article", "ICON_READ_UNREAD_ARTICLE", N_("Next _Unread Article"), "N", NULL, G_CALLBACK(do_read_next_unread_article) },
     { "read-next-article", NULL, N_("Next _Article"), "<control>N", NULL, G_CALLBACK(do_read_next_article) },
-    { "read-next-watched-article", NULL, N_("Next _Watched Article"), "<control><shift>N", N_("Next Watched Article"), G_CALLBACK(do_read_next_watched_article) },
-    { "read-next-unread-thread", "ICON_READ_UNREAD_THREAD", N_("Next Unread _Thread"), "T", N_("Next Unread Thread"), G_CALLBACK(do_read_next_unread_thread) },
-    { "read-next-thread", NULL, N_("Next Threa_d"), "<control>T", N_("Next Thread"), G_CALLBACK(do_read_next_thread) },
-    { "read-previous-article", NULL, N_("Pre_vious Article"), "V", N_("Previous Article"), G_CALLBACK(do_read_previous_article) },
-    { "read-previous-thread", NULL, N_("Previous _Thread"), "<control>V", N_("Previous Thread"), G_CALLBACK(do_read_previous_thread) },
-    { "read-parent-article", NULL, N_("_Parent Article"), "U", N_("Parent Article"), G_CALLBACK(do_read_parent_article) },
+    { "read-next-watched-article", NULL, N_("Next _Watched Article"), "<control><shift>N", NULL, G_CALLBACK(do_read_next_watched_article) },
+    { "read-next-unread-thread", "ICON_READ_UNREAD_THREAD", N_("Next Unread _Thread"), "T", NULL, G_CALLBACK(do_read_next_unread_thread) },
+    { "read-next-thread", NULL, N_("Next Threa_d"), "<control>T", NULL, G_CALLBACK(do_read_next_thread) },
+    { "read-previous-article", NULL, N_("Pre_vious Article"), "V", NULL, G_CALLBACK(do_read_previous_article) },
+    { "read-previous-thread", NULL, N_("Previous _Thread"), "<control>V", NULL, G_CALLBACK(do_read_previous_thread) },
+    { "read-parent-article", NULL, N_("_Parent Article"), "U", NULL, G_CALLBACK(do_read_parent_article) },
 
     { "plonk", NULL, N_("Ignore _Author"), NULL, NULL, G_CALLBACK(do_plonk) },
     { "watch-thread", NULL, N_("_Watch Thread"), NULL, NULL, G_CALLBACK(do_watch) },
     { "ignore-thread", NULL, N_("_Ignore Thread"), NULL, NULL, G_CALLBACK(do_ignore) },
-    { "view-article-score", "ICON_SCORE", N_("Edit Article's Watch/Ignore/Score ..."), "<control><shift>C", NULL, G_CALLBACK(do_show_score_dialog) },
-    { "add-article-score", "ICON_SCORE", N_("_Add a _Scoring Rule ..."), "S", NULL, G_CALLBACK(do_show_new_score_dialog) },
-    { "cancel-article", NULL, N_("Cance_l Article ..."), NULL, NULL, G_CALLBACK(do_cancel_article) },
-    { "supersede-article", NULL, N_("_Supersede Article ..."), NULL, NULL, G_CALLBACK(do_supersede_article) },
+    { "view-article-score", "ICON_SCORE", N_("Edit Article's Watch/Ignore/Score..."), "<control><shift>C", NULL, G_CALLBACK(do_show_score_dialog) },
+    { "add-article-score", "ICON_SCORE", N_("_Add a _Scoring Rule..."), "S", NULL, G_CALLBACK(do_show_new_score_dialog) },
+    { "cancel-article", NULL, N_("Cance_l Article..."), NULL, NULL, G_CALLBACK(do_cancel_article) },
+    { "supersede-article", NULL, N_("_Supersede Article..."), NULL, NULL, G_CALLBACK(do_supersede_article) },
     { "delete-article", GTK_STOCK_DELETE, N_("_Delete Article"), "Delete", NULL, G_CALLBACK(do_delete_article) },
-    { "mark-article-read", "ICON_ARTICLE_READ", N_("_Mark Article as Read"), "M", N_("Mark Article as Read"), G_CALLBACK(do_mark_article_read) },
-    { "mark-article-unread", "ICON_ARTICLE_UNREAD", N_("Mark Article as _Unread"), "<control>M", N_("Mark Article as Unread"), G_CALLBACK(do_mark_article_unread) },
+    { "clear-article-cache", NULL, N_("Clear Article Cache"), NULL, NULL, G_CALLBACK(do_clear_article_cache) },
+    { "mark-article-read", "ICON_ARTICLE_READ", N_("_Mark Article as Read"), "M", NULL, G_CALLBACK(do_mark_article_read) },
+    { "mark-article-unread", "ICON_ARTICLE_UNREAD", N_("Mark Article as _Unread"), "<control>M", NULL, G_CALLBACK(do_mark_article_unread) },
 
-    { "post", "ICON_COMPOSE_POST", N_("_Post to Newsgroup"), "P", N_("Post to Newsgroup"), G_CALLBACK(do_post) },
-    { "followup-to", "ICON_COMPOSE_FOLLOWUP", N_("_Followup to  Newsgroup"), "F", N_("Followup to Newsgroup"), G_CALLBACK(do_followup_to) },
-    { "reply-to", NULL, N_("_Reply to Author in Mail"), "R", N_("Reply to Author in Mail"), G_CALLBACK(do_reply_to) },
+    { "post", "ICON_COMPOSE_POST", N_("_Post to Newsgroup"), "P", NULL, G_CALLBACK(do_post) },
+    { "followup-to", "ICON_COMPOSE_FOLLOWUP", N_("_Followup to Newsgroup"), "F", NULL, G_CALLBACK(do_followup_to) },
+    { "reply-to", NULL, N_("_Reply to Author in Mail"), "R", NULL, G_CALLBACK(do_reply_to) },
 
     { "pan-web-page", NULL, N_("_Pan Home Page"), NULL, NULL, G_CALLBACK(do_pan_web) },
-    { "bug-report", NULL, N_("Give _Feedback or Report a Bug ..."), NULL, NULL, G_CALLBACK(do_bug_report) },
-    { "tip-jar", NULL, N_("_Tip Jar ..."), NULL, NULL, G_CALLBACK(do_tip_jar) },
+    { "bug-report", NULL, N_("Give _Feedback or Report a Bug..."), NULL, NULL, G_CALLBACK(do_bug_report) },
+    { "tip-jar", NULL, N_("_Tip Jar..."), NULL, NULL, G_CALLBACK(do_tip_jar) },
 #if GTK_CHECK_VERSION(2,6,0)
     { "about-pan", GTK_STOCK_ABOUT, N_("_About"), NULL, NULL, G_CALLBACK(do_about_pan) }
 #else
@@ -362,6 +364,8 @@ namespace
     { "show-text-markup",         NULL, N_("Show *Bold*, __Underlined__, and /Italic/"), NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "size-pictures-to-fit",     NULL, N_("Size Pictures to _Fit"),          NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
     { "monospace-font-enabled",   NULL, N_("Use _Monospace Font"),             "C", NULL, G_CALLBACK(prefs_toggle_callback_impl), false },
+    { "focus-on-image",           NULL, N_("Set Focus to Images"),            NULL, NULL, G_CALLBACK(prefs_toggle_callback_impl), true },
+
 
     { "work-online", "ICON_ONLINE", N_("_Work Online"), "L", NULL, G_CALLBACK(do_work_online), true },
     { "tabbed-layout", GTK_STOCK_JUMP_TO, N_("_Tabbed Layout"), "Z", NULL, G_CALLBACK(do_tabbed_layout), false },
@@ -369,13 +373,13 @@ namespace
     { "show-header-pane", NULL, N_("Show Hea_der Pane"), "<control>2", NULL, G_CALLBACK(do_show_header_pane), true },
     { "show-body-pane", NULL, N_("Show Bod_y Pane"), "<control>3", NULL, G_CALLBACK(do_show_body_pane), true },
     { "show-toolbar", NULL, N_("Show _Toolbar"), NULL, NULL, G_CALLBACK(do_show_toolbar), true },
-    { "shorten-group-names", GTK_STOCK_ZOOM_OUT, N_("Abbreviate _Names in Group Pane"), "B", N_("Abbreviate Newsgroup Names"), G_CALLBACK(do_shorten_group_names), false },
+    { "shorten-group-names", GTK_STOCK_ZOOM_OUT, N_("Abbreviate Group Names"), "B", NULL, G_CALLBACK(do_shorten_group_names), false },
 
-    { "match-only-unread-articles", "ICON_ONLY_UNREAD", N_("Match Only _Unread Articles"), NULL, N_("Match Only Unread Articles"), G_CALLBACK(do_match_only_unread_articles), false },
-    { "match-only-cached-articles", "ICON_ONLY_CACHED", N_("Match Only _Cached Articles"), NULL, N_("Match Only Cached Articles"), G_CALLBACK(do_match_only_cached_articles), false },
-    { "match-only-binary-articles", "ICON_ONLY_ATTACHMENTS", N_("Match Only _Complete Articles"), NULL, N_("Match Only Complete Articles"), G_CALLBACK(do_match_only_binary_articles), false },
-    { "match-only-my-articles", "ICON_ONLY_ME", N_("Match Only _My Articles"), NULL, N_("Match Only Articles Posted by Me"), G_CALLBACK(do_match_only_my_articles), false },
-    { "match-only-watched-articles", "ICON_ONLY_WATCHED", N_("Match Only _Watched Articles"), NULL, N_("Match Only Watched Articles"), G_CALLBACK(do_match_only_watched_articles), false },
+    { "match-only-unread-articles", "ICON_ONLY_UNREAD", N_("Match Only _Unread Articles"), NULL, NULL, G_CALLBACK(do_match_only_unread_articles), false },
+    { "match-only-cached-articles", "ICON_ONLY_CACHED", N_("Match Only _Cached Articles"), NULL, NULL, G_CALLBACK(do_match_only_cached_articles), false },
+    { "match-only-binary-articles", "ICON_ONLY_ATTACHMENTS", N_("Match Only _Complete Articles"), NULL, NULL, G_CALLBACK(do_match_only_binary_articles), false },
+    { "match-only-my-articles", "ICON_ONLY_ME", N_("Match Only _My Articles"), NULL, NULL, G_CALLBACK(do_match_only_my_articles), false },
+    { "match-only-watched-articles", "ICON_ONLY_WATCHED", N_("Match Only _Watched Articles"), NULL, NULL, G_CALLBACK(do_match_only_watched_articles), false },
 
     { "match-watched-articles", NULL, N_("Match Scores of 9999 (_Watched)"), NULL, NULL, G_CALLBACK(do_match_watched_articles), true },
     { "match-high-scoring-articles", NULL, N_("Match Scores of 5000...9998 (_High)"), NULL, NULL, G_CALLBACK(do_match_high_scoring_articles), true },
@@ -425,35 +429,14 @@ namespace
     ((GtkToggleActionCallback)(real_toggle_callbacks[name])) (a);
   }
 
-  GtkRadioActionEntry fallback_charsets[] =
+  void ensure_tooltip (GtkActionEntry * e)
   {
-    {"charset-ISO-8859-4",   NULL, N_("Baltic (ISO-8859-4)"), NULL, NULL, 0},
-    {"charset-ISO-8859-13",  NULL, N_("Baltic (ISO-8859-13)"), NULL, NULL, 1},
-    {"charset-windows-1257", NULL, N_("Baltic (Windows-1257)"), NULL, NULL, 2},
-    {"charset-ISO-8859-2",   NULL, N_("Central European (ISO-8859-2)"), NULL, NULL, 3},
-    {"charset-windows-1250", NULL, N_("Central European (Windows-1250)"), NULL, NULL, 4},
-    {"charset-gb2312",       NULL, N_("Chinese Simplified (gb2312)"), NULL, NULL, 5},
-    {"charset-big5",         NULL, N_("Chinese Traditional (big5)"), NULL, NULL, 6},
-    {"charset-ISO-8859-5",   NULL, N_("Cyrillic (ISO-8859-5)"), NULL, NULL, 7},
-    {"charset-windows-1251", NULL, N_("Cyrillic (Windows-1251)"), NULL, NULL, 8},
-    {"charset-KOI8-R",       NULL, N_("Cyrillic (KOI8-R)"), NULL, NULL, 9},
-    {"charset-KOI8-U",       NULL, N_("Cyrillic, Ukrainian (KOI8-U)"), NULL, NULL, 10},
-    {"charset-ISO-8859-7",   NULL, N_("Greek (ISO-8859-7)"), NULL, NULL, 11},
-    {"charset-ISO-2022-jp",  NULL, N_("Japanese (ISO-2022-jp)"), NULL, NULL, 12},
-    {"charset-euc-kr",       NULL, N_("Korean (euc-kr)"), NULL, NULL, 13},
-    {"charset-ISO-8859-9",   NULL, N_("Turkish (ISO-8859-9)"), NULL, NULL, 14},
-    {"charset-ISO-8859-1",   NULL, N_("Western (ISO-8859-1)"), NULL, NULL, 15},
-    {"charset-ISO-8859-15",  NULL, N_("Western, New (ISO-8859-15)"), NULL, NULL, 16},
-    {"charset-windows-1252", NULL, N_("Western (Windows-1252)"), NULL, NULL, 17},
-    {"charset-UTF-8",        NULL, N_("Unicode (UTF-8)"), NULL, NULL, 18}
-  };
-
-  void charset_cb (GtkAction *action, GtkRadioAction *current)
-  {
-    const int index (gtk_radio_action_get_current_value (current));
-    const char * name (fallback_charsets[index].name);
-    name += strlen("charset-");
-    pan_ui->do_set_charset (name);
+    if (!e->tooltip && e->label)
+    {
+      std::string s = e->label;
+      s.erase (std::remove (s.begin(), s.end(), '_'), s.end());
+      e->tooltip = g_strdup (s.c_str());
+    }
   }
 }
 
@@ -465,7 +448,10 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
 
   register_my_builtin_icons ();
 
-  for (GtkToggleActionEntry *it(toggle_entries), *end(it+n_toggle_entries); it!=end; ++it) {
+  for (GtkToggleActionEntry *it(toggle_entries), *end(it+n_toggle_entries); it!=end; ++it)
+  {
+    ensure_tooltip (reinterpret_cast<GtkActionEntry*>(it));
+
     it->is_active = prefs->get_flag (it->name, it->is_active);
     if (it->callback != G_CALLBACK(prefs_toggle_callback_impl)) {
       real_toggle_callbacks[it->name] = it->callback;
@@ -482,6 +468,9 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
     }
   }
 
+  for (GtkActionEntry *it(entries), *end(it+n_entries); it!=end; ++it)
+    ensure_tooltip (it);
+
   GtkActionGroup * action_group = _group = gtk_action_group_new ("Actions");
   gtk_action_group_set_translation_domain (action_group, NULL);
   gtk_action_group_add_actions (action_group, entries, n_entries, NULL);
@@ -490,20 +479,6 @@ pan :: add_actions (PanUI * ui, GtkUIManager * ui_manager, Prefs * p)
                                       match_toggle_entries, G_N_ELEMENTS(match_toggle_entries),
                                       show_mode,
                                       G_CALLBACK(show_matches_cb), NULL);
-  gtk_action_group_add_radio_actions (action_group,
-                                      fallback_charsets, G_N_ELEMENTS(fallback_charsets),
-                                      18, // UTF-8
-                                      G_CALLBACK(charset_cb), NULL);
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
-
-#if 1
-  int merge_id = gtk_ui_manager_new_merge_id (ui_manager);
-  char * path = "/main-window-menu/view-menu/character-encoding-menu/";
-  for (size_t i(0), n(G_N_ELEMENTS(fallback_charsets)); i!=n; ++i) {
-    const char * name (fallback_charsets[i].name);
-    gtk_ui_manager_add_ui (ui_manager, merge_id, 
-                           path, name, name, GTK_UI_MANAGER_MENUITEM, false);
-  }
-
-#endif
+  g_object_unref (G_OBJECT(action_group));
 }
