@@ -33,6 +33,7 @@ extern "C" {
 #include <pan/data/data.h>
 #include "group-pane.h"
 #include "pad.h"
+#include "sexy-icon-entry.h"
 
 using namespace pan;
 
@@ -53,7 +54,7 @@ namespace
     public:
       virtual void get_value (int column, GValue* setme) {
         switch (column) {
-          case COL_UNREAD: set_value_ulong   (setme, unread); break;
+          case COL_UNREAD: set_value_ulong (setme, unread); break;
         }
       }
   };
@@ -112,23 +113,21 @@ namespace
 }
 
 Quark
-GroupPane :: get_selection () const
+GroupPane :: get_first_selection () const
 {
-   Quark groupname;
+  Quark group;
 
-   GtkTreeIter iter;
-   GtkTreeModel * model;
-   GtkTreeSelection * selection (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
-   if (gtk_tree_selection_get_selected (selection, &model, &iter))
-     groupname = dynamic_cast<MyRow*>(_tree_store->get_row (&iter))->groupname;
+  const quarks_v groups (get_full_selection ());
+  if (!groups.empty())
+    group = groups.front();
 
-   return groupname;
+  return group;
 }
 
-quarks_t
+quarks_v
 GroupPane :: get_full_selection () const
 {
-  quarks_t groups;
+  quarks_v groups;
 
   // get a list of paths
   GtkTreeModel * model (0);
@@ -139,7 +138,7 @@ GroupPane :: get_full_selection () const
     GtkTreePath * path (static_cast<GtkTreePath*>(l->data));
     GtkTreeIter iter;
     if (gtk_tree_model_get_iter (model, &iter, path))
-      groups.insert (dynamic_cast<MyRow*>(_tree_store->get_row (&iter))->groupname);
+      groups.push_back (dynamic_cast<MyRow*>(_tree_store->get_row (&iter))->groupname);
   }
 
   // cleanup
@@ -234,9 +233,13 @@ GroupPane :: on_button_pressed (GtkWidget *treeview, GdkEventButton *event, gpoi
     GtkTreePath * path;
     if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW(treeview),
                                        (gint)event->x, (gint)event->y,
-                                       &path, NULL, NULL, NULL)) {
-      gtk_tree_selection_unselect_all (selection);
-      gtk_tree_selection_select_path (selection, path);
+                                       &path, NULL, NULL, NULL))
+    {
+      if (!gtk_tree_selection_path_is_selected (selection, path))
+      {
+        gtk_tree_selection_unselect_all (selection);
+        gtk_tree_selection_select_path (selection, path);
+      }
       gtk_tree_path_free (path);
     }
     do_popup_menu (treeview, event, userdata);
@@ -247,7 +250,8 @@ GroupPane :: on_button_pressed (GtkWidget *treeview, GdkEventButton *event, gpoi
            && (event->button == 1)
            && (event->send_event == false)
            && (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW(treeview)))
-           && !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK)))
+           && !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK))
+           && (pane->get_full_selection().size() == 1u))
   {
     GtkTreePath * path;
     GtkTreeViewColumn * col;
@@ -498,48 +502,39 @@ namespace
 void
 GroupPane :: set_filter (const std::string& search_text)
 {
-   GtkTreeView * view = GTK_TREE_VIEW(_tree_view);
+  GtkTreeView * view = GTK_TREE_VIEW(_tree_view);
 
-   // get the current selection
-   typedef std::set<GtkTreeRowReference*> sel_refs_t;
-   sel_refs_t sel_refs;
-   GtkTreeSelection * selection = gtk_tree_view_get_selection(view);
-   GList * tmp = gtk_tree_selection_get_selected_rows (selection, NULL);
-   GtkTreeModel * old_model = gtk_tree_view_get_model (view);
-   for (GList * l=tmp; l!=0; l=l->next) {
-      GtkTreePath * path = static_cast<GtkTreePath*>(l->data);
-      sel_refs.insert (gtk_tree_row_reference_new (old_model, path));
-      gtk_tree_path_free (path);
-   }
-   g_list_free (tmp);
+  // get the current selection
+  const quarks_v selected_groups (get_full_selection ());
 
-   // pmatch will point to a local TextMatch matching on the filter-phrase,
-   // or be a NULL pointer if the filter-phrase is empty
-   TextMatch match;
-   TextMatch * pmatch (0);
-   if (!search_text.empty()) {
-     match.set (search_text, TextMatch::CONTAINS, false);
-     pmatch = &match;
-   }
+  // pmatch will point to a local TextMatch matching on the filter-phrase,
+  // or be a NULL pointer if the filter-phrase is empty
+  TextMatch match;
+  TextMatch * pmatch (0);
+  if (!search_text.empty()) {
+    match.set (search_text, TextMatch::CONTAINS, false);
+    pmatch = &match;
+  }
 
-   // build and use the new store
-   tree_iters_t iters;
-   PanTreeStore * store = build_model (_data, pmatch, iters, _group_rows);
-   _tree_store = store;
-   gtk_tree_view_set_model(view, GTK_TREE_MODEL(store));
-   g_object_unref (G_OBJECT(store));
+  // build and use the new store
+  tree_iters_t iters;
+  PanTreeStore * store = build_model (_data, pmatch, iters, _group_rows);
+  _tree_store = store;
+  gtk_tree_view_set_model(view, GTK_TREE_MODEL(store));
+  g_object_unref (G_OBJECT(store));
 
-   // expand whatever paths need to be expanded
-   expand_iterators (iters, GTK_TREE_MODEL(store), GTK_TREE_VIEW(_tree_view));
+  // expand whatever paths need to be expanded
+  expand_iterators (iters, GTK_TREE_MODEL(store), GTK_TREE_VIEW(_tree_view));
 
-   // restore the selection
-   for (sel_refs_t::iterator it=sel_refs.begin(), end=sel_refs.end(); it!=end; ++it) {
-      GtkTreePath * path = gtk_tree_row_reference_get_path (*it);
-      gtk_tree_selection_select_path  (selection, path);
-      gtk_tree_path_free (path);
-      gtk_tree_row_reference_free (*it);
-   }
-   sel_refs.clear ();
+  // restore the selection
+  GtkTreeSelection * selection (gtk_tree_view_get_selection (view));
+  foreach_const (quarks_v, selected_groups, it) {
+    MyRow * row (find_row (*it));
+    if (row) {
+      GtkTreeIter iter = store->get_iter (row);
+      gtk_tree_selection_select_iter  (selection, &iter);
+    }
+  }
 }
 
 namespace
@@ -605,10 +600,10 @@ namespace
     remove_activate_soon_tag ();
   }
 
-  void reset_entry_filter_cb (GtkWidget * button, gpointer pane_gpointer)
+  void clear_button_clicked_cb (SexyIconEntry *e, SexyIconEntryPosition icon_pos, int button, gpointer pane_gpointer)
   {
-    GtkWidget * e = GTK_WIDGET (g_object_get_data (G_OBJECT(button), "entry"));
-    set_search_entry (e, "");
+    set_search_entry (GTK_WIDGET(e), "");
+    refresh_search_entry (GTK_WIDGET(e));
     search_text.clear ();
     search_entry_activated (NULL, pane_gpointer);
   }
@@ -630,9 +625,6 @@ namespace
   {
     search_text = gtk_entry_get_text (GTK_ENTRY(e));
     bump_activate_soon_tag (static_cast<GroupPane*>(pane_gpointer));
-
-    GtkWidget * b (GTK_WIDGET (g_object_get_data (G_OBJECT(e), "reset-button")));
-    gtk_widget_set_sensitive (b, !search_text.empty());
   }
 }
 
@@ -665,7 +657,7 @@ namespace
     std::string group_name (do_shorten ? get_short_name(StringView(name)) : name.to_string());
     if (is_g && unread) {
       char buf[64];
-      g_snprintf (buf, sizeof(buf), _(" (%lu)"), unread);
+      g_snprintf (buf, sizeof(buf), " (%lu)", unread);
       group_name += buf;
     }
     //if (unread || total) {
@@ -689,62 +681,97 @@ GroupPane :: set_name_collapse (bool b)
 ****
 ***/
 
-namespace
+/**
+ * Walk circularly through the subscribed groups,
+ * looking for the next one after our current
+ * selection, possibly only ones with unread messages.
+ */
+GtkTreePath*
+GroupPane :: find_next_subscribed_group (bool unread_only)
 {
-  struct NextGroupWalk: public PanTreeStore::WalkFunctor
+  GtkTreeView * view (GTK_TREE_VIEW(_tree_view));
+  GtkTreeModel * model (gtk_tree_view_get_model (view));
+
+  // find how many subscribed groups the tree's got
+  GtkTreeIter sub_iter;
+  gtk_tree_model_iter_nth_child (model, &sub_iter, 0, 0); // 'sub' node
+  const int n_groups = gtk_tree_model_iter_n_children (model, &sub_iter);
+  if (n_groups < 1)
+    return 0;
+
+  // find the index of the current selection
+  // (or -1 if no subscribed groups are selected)
+  int start_pos = -1;
+  GtkTreeIter sel_iter;
+  const Quark group = get_first_selection ();
+  if (!group.empty()) {
+    const MyRow* row = find_row (group);
+    sel_iter = PAN_TREE_STORE(model)->get_iter (row);
+    GtkTreePath * path = gtk_tree_model_get_path (model, &sel_iter);
+    gint depth = gtk_tree_path_get_depth (path);
+    gint* indices = gtk_tree_path_get_indices (path);
+    if (depth==2 && indices[0]==0) // a subscribed group is selected
+      start_pos = indices[1];
+    gtk_tree_path_free (path);
+  }
+
+  // loop through the `sub' node's children looking for
+  // groups that match our criteria.
+  int n = start_pos;
+  for (;;)
   {
-    const ActionManager& am;
-    GtkTreeSelection * sel;
-    const Quark current_group;
-    bool current_group_reached;
-    const bool unread_only;
-
-    NextGroupWalk (const ActionManager& a, GtkTreeSelection * s, const Quark& g, bool unread):
-      am (a),
-      sel (s),
-      current_group(g),  
-      current_group_reached (g.empty()),
-      unread_only (unread) { }
-    virtual ~NextGroupWalk () { }
-
-    virtual bool operator()(PanTreeStore * store, PanTreeStore::Row * r,
-                            GtkTreeIter * iter, GtkTreePath * unused)
-    {
-      const MyRow * row (dynamic_cast<MyRow*>(r));
-      const Quark& name (row->groupname);
-      if (is_group (name)) {
-        if (current_group_reached && (row->unread || !unread_only)) {
-          gtk_tree_selection_unselect_all (sel);
-          gtk_tree_selection_select_iter (sel, iter);
-          am.activate_action ("read-selected-group");
-          return false; // done
-        }
-        if (name == current_group)
-          current_group_reached = true;
-      }
-      return true; // keep marching
+    if (++n == n_groups) {
+      if (start_pos == -1)
+        return 0;
+      n = 0;
     }
-  };
+    if (n == start_pos)
+      return 0;
+
+    GtkTreeIter group_iter;
+    gtk_tree_model_iter_nth_child (model, &group_iter, &sub_iter, n);
+    const MyRow * row (dynamic_cast<MyRow*>(_tree_store->get_row (&group_iter)));
+    if (!unread_only || row->unread)
+      return gtk_tree_model_get_path (model, &group_iter);
+  }
 }
 
 void
-GroupPane :: read_next_unread_group ()
+GroupPane :: read_group (GtkTreePath * path)
 {
+  g_assert (path);
+
   GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
   GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
-  const Quark current_group (get_selection ());
-  NextGroupWalk walker (_action_manager, sel, current_group, true);
-  _tree_store->walk (walker);
+
+  gtk_tree_view_expand_row (view, path, true);
+  gtk_tree_view_expand_to_path (view, path);
+  gtk_tree_view_set_cursor (view, path, NULL, false);
+  gtk_tree_view_scroll_to_cell (view, path, NULL, true, 0.5f, 0.0f);
+  gtk_tree_selection_unselect_all (sel);
+  gtk_tree_selection_select_path (sel, path);
+  _action_manager.activate_action ("read-selected-group");
+}
+
+void
+GroupPane :: read_next_group_impl (bool unread_only)
+{
+  GtkTreePath * path = find_next_subscribed_group (unread_only);
+  if (path) {
+    read_group (path);
+    gtk_tree_path_free (path);
+  }
+}
+void
+GroupPane :: read_next_unread_group ()
+{
+  read_next_group_impl (true);
 }
 
 void
 GroupPane :: read_next_group ()
 {
-  GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
-  GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
-  const Quark current_group (get_selection ());
-  NextGroupWalk walker (_action_manager, sel, current_group, false);
-  _tree_store->walk (walker);
+  read_next_group_impl (false);
 }
 
 /***
@@ -772,42 +799,28 @@ namespace
 GtkWidget*
 GroupPane :: create_filter_entry ()
 {
-  GtkWidget * h = gtk_hbox_new (false, 0);
-  GtkWidget * entry = gtk_entry_new ();
-  gtk_widget_set_size_request (entry, 100, -1);
+  GtkWidget * entry = sexy_icon_entry_new ();
+
+  GtkWidget * icon = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
+  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, GTK_IMAGE(icon));
+  sexy_icon_entry_set_icon_highlight(SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, true);
+  gtk_widget_set_size_request (entry, 133, -1);
+
   _action_manager.disable_accelerators_when_focused (entry);
   g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
   g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), NULL);
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
+  g_signal_connect (entry, "icon_released", G_CALLBACK(clear_button_clicked_cb), this);
   entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed_by_user), this);
   refresh_search_entry (entry);
-  gtk_box_pack_start (GTK_BOX(h), entry, 0, 0, false);
-
-  GtkTooltips * tips = gtk_tooltips_new ();
-  GtkWidget * w = gtk_button_new ();
-  g_object_set_data (G_OBJECT(w), "entry", entry);
-  g_object_set_data (G_OBJECT(entry), "reset-button", w);
-  gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
-  g_signal_connect (w, "clicked", G_CALLBACK(reset_entry_filter_cb), this);
-  GtkWidget * image = gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON);
-  gtk_container_add (GTK_CONTAINER(w), image);
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(tips), w, _("Clear the Filter"), NULL);
-  gtk_box_pack_start (GTK_BOX(h), w, 0, 0, false);
-  gtk_widget_set_sensitive (w, false);
-
-  // when the button's destroyed, free the tips too
-  g_object_ref (tips);
-  gtk_object_sink (GTK_OBJECT (tips));
-  g_signal_connect_swapped (w, "destroy", G_CALLBACK(g_object_unref), tips);
-
-  return h;
+  return entry;
 }
 
 void
 GroupPane :: on_selection_changed (GtkTreeSelection * sel, gpointer pane_gpointer)
 {
   GroupPane * self (static_cast<GroupPane*>(pane_gpointer));
-  Quark group (self->get_selection());
+  Quark group (self->get_first_selection());
   self->_action_manager.sensitize_action ("show-group-preferences-dialog", is_group(group));
 }
 
@@ -822,8 +835,6 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   sub_title_quark = new Quark (_("Subscribed Groups"));
   other_title_quark = new Quark (_("Other Groups"));
 
-  quarks_t groups;
-
   // build this first because _tree_view is needed for a callback...
   tree_iters_t iters;
   _tree_store = build_model (_data, NULL, iters, _group_rows);
@@ -831,9 +842,13 @@ GroupPane :: GroupPane (ActionManager& action_manager, Data& data, Prefs& prefs)
   g_object_unref (G_OBJECT(_tree_store)); // will die with the view
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW(_tree_view), false);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(_tree_view), false);
+#if GTK_CHECK_VERSION(2,10,0)
+  gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW(_tree_view), true);
+#endif
   expand_iterators (iters, GTK_TREE_MODEL(_tree_store), GTK_TREE_VIEW(_tree_view));
 
   GtkTreeSelection * selection (gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view)));
+  gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
   gtk_tree_selection_set_select_function (selection, select_func, 0, 0);
   g_signal_connect (selection, "changed", G_CALLBACK(on_selection_changed), this);
   on_selection_changed (selection, this);
