@@ -20,13 +20,15 @@
 #include <config.h>
 #include <map>
 #include <string>
+#include <sstream>
 extern "C" {
   #include <glib/gi18n.h>
 }
 #include <pan/general/debug.h>
+#include <pan/general/e-util.h>
 #include <pan/general/file-util.h>
 #include <pan/general/foreach.h>
-#include <pan/data/scorefile.h>
+#include <pan/usenet-utils/scorefile.h>
 #include <pan/tasks/task-article.h>
 #include <pan/tasks/task-groups.h>
 #include <pan/tasks/task-xover.h>
@@ -34,6 +36,7 @@ extern "C" {
 #include <pan/icons/pan-pixbufs.h>
 #include "actions.h"
 #include "body-pane.h"
+#include "e-charset-picker.h"
 #include "dl-headers-ui.h"
 #include "group-pane.h"
 #include "group-prefs-dialog.h"
@@ -55,6 +58,20 @@ extern "C" {
 #include "task-pane.h"
 #include "url.h"
 
+namespace pan
+{
+  void
+  g_object_ref_sink_pan (GObject * o)
+  {
+#if GLIB_CHECK_VERSION(2,10,0)
+    g_object_ref_sink (o);
+#else
+    g_object_ref (o);
+    gtk_object_sink (GTK_OBJECT(o));
+#endif
+  }
+}
+
 using namespace pan;
 
 namespace
@@ -80,10 +97,17 @@ namespace
 
   void set_visible (GtkWidget * w, bool visible)
   {
-    if (visible)
-      gtk_widget_show (w);
-    else
+    if (!visible)
       gtk_widget_hide (w);
+    else if (GTK_IS_WINDOW(w))
+      gtk_window_present (GTK_WINDOW(w));
+    else
+      gtk_widget_show (w);
+  }
+
+  void toggle_visible (GtkWidget * w)
+  {
+    set_visible (w, !GTK_WIDGET_VISIBLE(w));
   }
 }
 
@@ -155,18 +179,12 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   _taskbar (0),
   _ttips (gtk_tooltips_new ())
 {
+  g_object_ref_sink_pan (G_OBJECT(_ttips));
+  g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, _ttips);
+
   char * filename = g_build_filename (file::get_pan_home().c_str(), "pan.ui", NULL);
-#if 0
-  GError * gerr (0);
-  gtk_ui_manager_add_ui_from_file (_ui_manager, filename, &gerr);
-  if (gerr)  {
-    Log::add_err (gerr->message);
-    g_clear_error (&gerr);
-  }
-#else
   if (!gtk_ui_manager_add_ui_from_file (_ui_manager, filename, NULL))
     gtk_ui_manager_add_ui_from_string (_ui_manager, fallback_ui_file, -1, NULL);
-#endif
   g_free (filename);
   g_signal_connect (_ui_manager, "add_widget", G_CALLBACK (add_widget), this);
   add_actions (this, _ui_manager, &prefs);
@@ -176,7 +194,7 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
 
   //_group_pane = new GroupPane (*this, data, _prefs);
   _group_pane = new GroupPane (*this, data, _prefs);
-  _header_pane = new HeaderPane (*this, data, _queue, _cache, _prefs);
+  _header_pane = new HeaderPane (*this, data, _queue, _cache, _prefs, *this);
   _body_pane = new BodyPane (data, _cache, _prefs);
 
   std::string path = "/ui/main-window-toolbar";
@@ -297,11 +315,18 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
 
   _prefs.add_listener (this);
 
-  g_object_ref (_ttips);
-  gtk_object_sink (GTK_OBJECT(_ttips));
-  g_signal_connect_swapped (_root, "destroy", G_CALLBACK(g_object_unref), _ttips);
-
   gtk_accel_map_load (get_accel_filename().c_str());
+
+  { // make sure taskbar views have the right tasks in them -- 
+    // when Pan first starts, the active tasks are already running
+    Queue::task_states_t task_states;
+    queue.get_all_task_states(task_states);    
+    foreach(Queue::tasks_t, task_states.tasks, it) {
+      Queue::TaskState s = task_states.get_state(*it);
+      if (s == Queue::RUNNING || s == Queue::DECODING)
+        on_queue_task_active_changed (queue, *(*it), true);
+    }
+  }
 }
 
 namespace
@@ -312,7 +337,9 @@ namespace
 
 GUI :: ~GUI ()
 {
-  gtk_accel_map_save (get_accel_filename().c_str());
+  const std::string accel_filename (get_accel_filename());
+  gtk_accel_map_save (accel_filename.c_str());
+  chmod (accel_filename.c_str(), 0600);
 
   if (hpane)
     _prefs.set_int ("main-window-hpane-position", gtk_paned_get_position(GTK_PANED(hpane)));
@@ -343,6 +370,7 @@ GUI :: ~GUI ()
 
   foreach (std::set<GtkWidget*>, unref, it)
     gtk_widget_unref (*it);
+  g_object_unref (G_OBJECT(_ui_manager));
 }
 
 /***
@@ -475,32 +503,33 @@ GUI :: disable_accelerators_when_focused (GtkWidget * w) const
 namespace
 {
   static std::string prev_path;
+}
 
-  std::string get_save_attachments_path_from_user (GtkWindow * parent, const Prefs& prefs)
-  {
-    if (prev_path.empty())
-      prev_path = prefs.get_string ("default-save-attachments-path", g_get_home_dir ());
-    if (!file :: file_exists (prev_path.c_str()))
-      prev_path = g_get_home_dir ();
+std::string
+GUI :: prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs)
+{
+  if (prev_path.empty())
+    prev_path = prefs.get_string ("default-save-attachments-path", g_get_home_dir ());
+  if (!file :: file_exists (prev_path.c_str()))
+    prev_path = g_get_home_dir ();
 
-    GtkWidget * w = gtk_file_chooser_dialog_new (_("Save Attachments"), parent,
-                                                 GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                                 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-                                                 NULL);
-    gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_ACCEPT);
-    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(w), prev_path.c_str());
-    const int response (gtk_dialog_run (GTK_DIALOG(w)));
-    std::string path;
-    if (response == GTK_RESPONSE_ACCEPT) {
-      char * tmp = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
-      prev_path = path = tmp;
-      g_free (tmp);
-    }
-
-    gtk_widget_destroy(w);
-    return path;
+  GtkWidget * w = gtk_file_chooser_dialog_new (_("Save NZB's Files"), parent,
+                                               GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                               GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                                               NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_ACCEPT);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(w), prev_path.c_str());
+  const int response (gtk_dialog_run (GTK_DIALOG(w)));
+  std::string path;
+  if (response == GTK_RESPONSE_ACCEPT) {
+    char * tmp = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (w));
+    prev_path = path = tmp;
+    g_free (tmp);
   }
+
+  gtk_widget_destroy(w);
+  return path;
 }
 
 void GUI :: do_save_articles ()
@@ -531,11 +560,22 @@ void GUI :: do_import_tasks ()
 {
   // get a list of files to import
   GtkWidget * dialog = gtk_file_chooser_dialog_new (
-    _("Import NZB File(s)"), get_window(_root),
+    _("Import NZB Files"), get_window(_root),
     GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
     NULL);
+
+  GtkFileFilter * filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*.nzb");
+  gtk_file_filter_set_name (filter, _("NZB Files"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*");
+  gtk_file_filter_set_name (filter, _("All Files"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
+
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(dialog), true);
   typedef std::vector<std::string> strings_t;
   strings_t filenames;
@@ -550,9 +590,9 @@ void GUI :: do_import_tasks ()
   gtk_widget_destroy (dialog);
 
   // if we're importing files, build the tasks...
-  std::vector<Task*> tasks;
+  Queue::tasks_t tasks;
   if (!filenames.empty()) {
-    const std::string path (get_save_attachments_path_from_user (get_window(_root), _prefs));
+    const std::string path (prompt_user_for_save_path (get_window(_root), _prefs));
     if (!path.empty())
       foreach_const (strings_t, filenames, it)
         NZB :: tasks_from_nzb_file (*it, path, _cache, _data, _data, _data, tasks);
@@ -574,13 +614,14 @@ namespace
 void GUI :: do_show_task_window ()
 {
   static TaskPane * task_pane (0);
-  if (task_pane)
-    gtk_window_present (GTK_WINDOW(task_pane->root()));
-  else {
+
+  if (!task_pane) {
     task_pane = new TaskPane (_queue, _prefs);
-    g_signal_connect (task_pane->root(), "destroy", G_CALLBACK(task_pane_destroyed_cb), &task_pane);
-    gtk_widget_show (task_pane->root());
+    g_signal_connect (task_pane->root(), "destroy",
+                      G_CALLBACK(task_pane_destroyed_cb), &task_pane);
   }
+
+  toggle_visible (task_pane->root());
 }
 
 namespace
@@ -603,18 +644,11 @@ void GUI :: on_log_entry_added (const Log::Entry& e)
     set_bin_child (_event_log_button, _error_image);
 
   if (_queue.is_online() && (e.severity & Log::PAN_SEVERITY_URGENT)) {
-    std::string msg (e.message);
-    if (msg.find ("ENOSPC") != e.message.npos) {
-      msg.erase (msg.find ("ENOSPC"), 6);
-      msg += ' ';
-      msg += _("Pan is now offline. Please ensure that space is available, then use File|Work Online to continue.");
-      toggle_action ("work-online", false);
-    }
     GtkWidget * w = gtk_message_dialog_new (get_window(_root),
                                             GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
                                             GTK_MESSAGE_ERROR,
                                             GTK_BUTTONS_CLOSE,
-                                            "%s", msg.c_str());
+                                            "%s", e.message.c_str());
     g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
     gtk_widget_show_all (w);
   }
@@ -622,13 +656,15 @@ void GUI :: on_log_entry_added (const Log::Entry& e)
 
 void GUI :: do_show_log_window ()
 {
-  static GtkWidget * log_window (0);
-  if (!log_window) {
-    log_window = log_dialog_new (_prefs, get_window (_root));
-    g_signal_connect (log_window, "destroy", G_CALLBACK(gtk_widget_destroyed), &log_window);
-  }
-  gtk_window_present (GTK_WINDOW(log_window));
   set_bin_child (_event_log_button, _info_image);
+
+  static GtkWidget * w (0);
+  if (!w) {
+    w = log_dialog_new (_prefs, get_window (_root));
+    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
+  }
+
+  toggle_visible (w);
 }
 void GUI :: do_select_all_articles ()
 {
@@ -661,9 +697,8 @@ void GUI :: do_show_preferences_dialog ()
 }
 void GUI :: do_show_group_preferences_dialog ()
 {
-  const Quark group (_group_pane->get_selection ());
-  if (!group.empty())
-  {
+  const Quark group (_group_pane->get_first_selection ());
+  if (!group.empty()) {
     GroupPrefsDialog * dialog = new GroupPrefsDialog (_data, group, _group_prefs, get_window(_root));
     gtk_widget_show (dialog->root());
   }
@@ -706,8 +741,14 @@ void GUI :: do_rot13_selected_text ()
 void GUI :: on_progress_finished (Progress & p, int status)
 {
   TaskArticle * ta = dynamic_cast<TaskArticle*>(&p);
-  if (status==OK && ta && ta->get_article().message_id == latest_read_article.message_id)
-    _body_pane->set_article (latest_read_article);
+  if (status==OK && ta)
+  {
+    _body_pane->set_article (ta->get_article());
+
+    // maybe update the tab
+    if (is_action_active ("tabbed-layout"))
+      activate_action ("jump-to-body-tab");
+  }
 }
 
 void GUI :: do_read_selected_article ()
@@ -715,24 +756,21 @@ void GUI :: do_read_selected_article ()
   const Article* article (_header_pane->get_first_selected_article ());
   if (article)
   {
-    latest_read_article = *article;
-    Task * t = new TaskArticle (_data, _data, *article, _cache, _data, this);
-    _queue.add_task (t, Queue::TOP);
-
-    // expand its thread in the header pane
     _header_pane->expand_selected ();
 
-    // maybe update the tab
-    if (is_action_active ("tabbed-layout"))
-      activate_action ("jump-to-body-tab");
+    Task * t = new TaskArticle (_data, _data, *article, _cache, _data, this);
+    _queue.add_task (t, Queue::TOP);
   }
 }
 void GUI :: do_download_selected_article ()
 {
-  typedef std::set<const Article*> article_set_t;
-  const article_set_t articles (_header_pane->get_full_selection ());
-  foreach_const (article_set_t, articles, it)
-    _queue.add_task (new TaskArticle (_data, _data, **it, _cache, _data, this));
+  typedef std::vector<const Article*> article_vector_t;
+  const article_vector_t articles (_header_pane->get_full_selection_v ());
+  Queue::tasks_t tasks;
+  foreach_const (article_vector_t, articles, it)
+    tasks.push_back (new TaskArticle (_data, _data, **it, _cache, _data));
+  if (!tasks.empty())
+    _queue.add_tasks (tasks, Queue::TOP);
 }
 void GUI :: do_clear_header_pane ()
 {
@@ -885,7 +923,7 @@ GUI :: score_add (int mode)
 {
   Quark group (_header_pane->get_group());
   if (group.empty())
-    group = _group_pane->get_selection();
+    group = _group_pane->get_first_selection();
 
   Article a;
   const Article* article (_header_pane->get_first_selected_article ());
@@ -931,6 +969,7 @@ void GUI :: do_supersede_article ()
   g_mime_message_set_sender (new_message, g_mime_message_get_sender (message));
   g_mime_message_set_subject (new_message, g_mime_message_get_subject (message));
   g_mime_message_set_header (new_message, "Newsgroups", g_mime_message_get_header (message, "Newsgroups"));
+  g_mime_message_set_header (new_message, "References", g_mime_message_get_header (message, "References"));
   if ((cpch = g_mime_message_get_reply_to (message)))
               g_mime_message_set_reply_to (new_message, cpch);
   if ((cpch = g_mime_message_get_header (message,     "Followup-To")))
@@ -1039,6 +1078,12 @@ void GUI :: do_delete_article ()
     if ((*it)->message_id == mid)
       _body_pane->clear ();
 }
+
+void GUI :: do_clear_article_cache ()
+{
+  _cache.clear ();
+}
+
 void GUI :: do_mark_article_read ()
 {
   const std::set<const Article*> article_set (_header_pane->get_nested_selection ());
@@ -1061,7 +1106,7 @@ GUI :: do_post ()
   // add newsgroup...
   Quark group (_header_pane->get_group ());
   if (group.empty())
-    group = _group_pane->get_selection ();
+    group = _group_pane->get_first_selection ();
   if (!group.empty())
     g_mime_message_add_header (message, "Newsgroups", group.c_str());
 
@@ -1114,13 +1159,13 @@ void GUI :: do_tip_jar ()
 void GUI :: do_about_pan ()
 {
 #if GTK_CHECK_VERSION(2,6,0)
-  const gchar * authors [] = { "Charles Kerr", 0 };
+  const gchar * authors [] = { "Charles Kerr <charles@rebelbase.com>", "Calin Culianu <calin@ajvar.org> - Threaded Decoding", 0 };
   GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
   GtkAboutDialog * w (GTK_ABOUT_DIALOG (gtk_about_dialog_new ()));
   gtk_about_dialog_set_name (w, _("Pan"));
   gtk_about_dialog_set_version (w, PACKAGE_VERSION);
   gtk_about_dialog_set_comments (w, VERSION_TITLE);
-  gtk_about_dialog_set_copyright (w, _("Copyright © 2002-2006 Charles Kerr"));
+  gtk_about_dialog_set_copyright (w, _("Copyright © 2002-2007 Charles Kerr"));
   gtk_about_dialog_set_website (w, "http://pan.rebelbase.com/");
   gtk_about_dialog_set_logo (w, logo);
   gtk_about_dialog_set_license (w, LICENSE);
@@ -1140,7 +1185,7 @@ void GUI :: do_about_pan ()
   gtk_box_pack_start (box, gtk_image_new_from_pixbuf (logo), false, false, PAD);
   gtk_box_pack_start (box, gtk_label_new("Pan " PACKAGE_VERSION), false, false, PAD);
   gtk_box_pack_start (box, gtk_label_new(VERSION_TITLE), false, false, 0);
-  gtk_box_pack_start (box, gtk_label_new(_("Copyright © 2002-2006 Charles Kerr")), false, false, 0);
+  gtk_box_pack_start (box, gtk_label_new(_("Copyright © 2002-2007 Charles Kerr")), false, false, 0);
   gtk_box_pack_start (box, gtk_label_new("http://pan.rebelbase.com/"), false, false, PAD);
   gtk_widget_show_all (dialog);
   g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
@@ -1201,8 +1246,28 @@ namespace
       w = NULL;
     return w;
   }
+
+  gboolean grab_focus_idle (gpointer w)
+  {
+    gtk_widget_grab_focus (GTK_WIDGET(w));
+    return false;
+  }
 }
 
+void
+GUI :: notebook_page_switched_cb (GtkNotebook * notebook, GtkNotebookPage * page, gint page_num, gpointer user_data)
+{
+  GUI * gui = static_cast<GUI*>(user_data);
+  GtkWidget * w;
+  switch (page_num) {
+    case 0: w = gui->_group_pane->get_default_focus_widget(); break;
+    case 1: w = gui->_header_pane->get_default_focus_widget(); break;
+    case 2: w = gui->_body_pane->get_default_focus_widget(); break;
+    default: g_assert (0 && "invalid page reached!"); break;
+  }
+  g_idle_add (grab_focus_idle, w);
+}
+ 
 void GUI :: do_tabbed_layout (bool tabbed)
 {
   if (hpane) {
@@ -1240,6 +1305,7 @@ void GUI :: do_tabbed_layout (bool tabbed)
     gtk_notebook_append_page (n, header_w, gtk_label_new_with_mnemonic (_("_2. Header Pane")));
     gtk_notebook_append_page (n, body_w, gtk_label_new_with_mnemonic (_("_3. Body Pane")));
     gtk_notebook_set_tab_border (n, PAD_SMALL);
+    g_signal_connect (n, "switch-page", G_CALLBACK(notebook_page_switched_cb), this);
   }
   else
   {
@@ -1311,29 +1377,78 @@ void GUI :: do_show_matches (const Data::ShowType show_type)
 {
   _header_pane->set_show_type (show_type);
 }
+
+void GUI :: do_show_selected_article_info ()
+{
+  const Article* a = _header_pane->get_first_selected_article ();
+
+  if (a != 0)
+  {
+    // date
+    EvolutionDateMaker date_maker;
+    char * date = date_maker.get_date_string (a->time_posted);
+
+    // article parts
+    std::vector<int> missing_parts;
+    int i = 0;
+    foreach_const (Article::parts_t, a->parts, it) {
+      ++i;
+      if (it->empty())
+        missing_parts.push_back (i);
+    }
+    char msg[512];
+    *msg = '\0';
+    std::ostringstream s;
+    if (i > 1) {
+      if (missing_parts.empty())
+        g_snprintf (msg, sizeof(msg), _("This article has all %d parts."), i);
+      else
+        g_snprintf (msg, sizeof(msg), _("This article is missing %d of its %d parts:"), (int)missing_parts.size(), i);
+      foreach_const (std::vector<int>, missing_parts, it)
+        s << ' ' << *it;
+    }
+
+    GtkWidget * w = gtk_message_dialog_new_with_markup (
+      get_window(_root),
+      GTK_DIALOG_DESTROY_WITH_PARENT,
+      GTK_MESSAGE_INFO,
+      GTK_BUTTONS_CLOSE,
+        "<b>%s</b>: %s\n" "<b>%s</b>: %s\n"
+        "<b>%s</b>: %s\n" "<b>%s</b>: %s\n"
+        "<b>%s</b>: %lu\n" "<b>%s</b>: %lu\n"
+        "\n"
+        "%s" "%s",
+        _("Subject"), a->subject.c_str(), _("From"), a->author.c_str(),
+        _("Date"), date, _("Message-ID"), a->message_id.c_str(),
+        _("Lines"), a->get_line_count(), _("Bytes"), a->get_byte_count(),
+        msg, s.str().c_str());
+    g_signal_connect_swapped (w, "response", G_CALLBACK (gtk_widget_destroy), w);
+    gtk_widget_show_all (w);
+
+    // cleanup
+    g_free (date);
+  }
+}
+
 void GUI :: do_quit ()
 {
   gtk_main_quit ();
 }
 void GUI :: do_read_selected_group ()
 {
-  const Quark group (_group_pane->get_selection ());
+  const Quark group (_group_pane->get_first_selection ());
 
   // update the titlebar
-  if (group.empty())
-    gtk_window_set_title (get_window(_root), _("Pan"));
-  else {
-    char * buf = g_strdup_printf (_("Pan: %s"), group.c_str());
-    gtk_window_set_title (get_window(_root), buf);
-    g_free (buf);
+  std::string s (_("Pan"));
+  if (!group.empty()) {
+    s += ": ";
+    s += group.c_str();
   }
+  gtk_window_set_title (get_window(_root), s.c_str());
 
   // set the charset encoding based upon that group's default
-  if (!group.empty()) {
-    std::string charset (_group_prefs.get_string (group, "character-encoding", "UTF-8"));
-    charset.insert (0, "charset-");
-    toggle_action (charset.c_str(), true);
-  }
+  if (!group.empty())
+    set_charset (_group_prefs.get_string (group, "character-encoding", "UTF-8"));
 
   // update the header pane
   watch_cursor_on ();
@@ -1369,21 +1484,22 @@ void GUI :: do_read_selected_group ()
 }
 void GUI :: do_mark_selected_groups_read ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach_const (quarks_t, group_names, it)
+  const quarks_v group_names (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, group_names, it)
     _data.mark_group_read (*it);
 }
 void GUI :: do_clear_selected_groups ()
 {
-  Quark group (_group_pane->get_selection());
-  _data.group_clear_articles (group);
+  const quarks_v groups (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, groups, it)
+    _data.group_clear_articles (*it);
 }
 
 void GUI :: do_xover_selected_groups ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
+  const quarks_v groups (_group_pane->get_full_selection ());
   const bool mark_read (_prefs.get_flag ("mark-group-read-before-xover", false));
-  foreach_const (quarks_t, group_names, it) {
+  foreach_const (quarks_v, groups, it) {
     if (mark_read)
       _data.mark_group_read (*it);
     _queue.add_task (new TaskXOver (_data, *it, TaskXOver::NEW), Queue::TOP);
@@ -1405,13 +1521,14 @@ void GUI :: do_xover_subscribed_groups ()
 
 void GUI :: do_download_headers ()
 {
-  const quarks_t groups (_group_pane->get_full_selection ());
+  const quarks_v groups_v (_group_pane->get_full_selection ());
+  const quarks_t groups (groups_v.begin(), groups_v.end());
   headers_dialog (_data, _prefs, _queue, groups, get_window(_root));
 }
 
 void GUI :: do_refresh_groups ()
 {
-  std::vector<Task*> tasks;
+  Queue::tasks_t tasks;
 
   const quarks_t servers (_data.get_servers ());
   foreach_const_r (quarks_t, servers, it)
@@ -1423,20 +1540,32 @@ void GUI :: do_refresh_groups ()
 
 void GUI :: do_subscribe_selected_groups ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach (quarks_t, group_names, it)
+  const quarks_v group_names (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, group_names, it)
     _data.set_group_subscribed (*it, true);
 }
 void GUI :: do_unsubscribe_selected_groups ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach (quarks_t, group_names, it)
+  const quarks_v groups (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, groups, it)
     _data.set_group_subscribed (*it, false);
 }
-void GUI :: do_set_charset (const char * charset)
+void GUI :: do_prompt_for_charset ()
 {
-  _fallback_charset = charset;
-  _body_pane->set_character_encoding (charset);
+  if (_charset.empty())
+      _charset = "UTF-8";
+
+  char * tmp = e_charset_picker_dialog (_("Character Encoding"),
+                                        _("Body Pane Encoding"),
+                                        _charset.c_str(),
+                                        get_window(root()));
+  set_charset (tmp);
+  free (tmp);
+}
+void GUI :: set_charset (const StringView& s)
+{
+  _charset.assign (s.str, s.len);
+  _body_pane->set_character_encoding (_charset.c_str());
 }
 
 /***
@@ -1537,7 +1666,7 @@ GUI :: set_queue_size_label (unsigned int running,
   if (!size)
     g_snprintf (str, sizeof(str), _("No Tasks"));
   else
-    g_snprintf (str, sizeof(str), _("Tasks: %u/%u"), running, size);
+    g_snprintf (str, sizeof(str), "%s: %u/%u", _("Tasks"), running, size);
 
   // update the gui
   gtk_label_set_text (GTK_LABEL(_queue_size_label), str);
@@ -1583,12 +1712,15 @@ GUI :: on_queue_error (Queue& q, const StringView& message)
 {
   if (_queue.is_online())
   {
-    toggle_action ("work-online", false);
-
-    std::string s (message);
-    s += "\n \n";
+    std::string s;
+    if (!message.empty()) {
+      s.assign (message.str, message.len);
+      s += "\n \n";
+    }
     s += _("Pan is now offline. Please see \"File|Event Log\" and correct the problem, then use \"File|Work Online\" to continue.");
     Log::add_urgent_va ("%s", s.c_str());
+
+    toggle_action ("work-online", false);
   }
 }
 
