@@ -17,29 +17,26 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <config.h>
-#include <cassert>
-#include <cctype>
-#include <cmath>
-#include <algorithm>
-#include <functional>
 extern "C" {
+  #include <config.h>
   #include <glib/gi18n.h>
   #include <gtk/gtk.h>
 }
+#include <cctype>
+#include <cmath>
+#include <algorithm>
 #include <pan/general/debug.h>
 #include <pan/general/e-util.h>
 #include <pan/general/foreach.h>
 #include <pan/general/quark.h>
 #include <pan/general/log.h>
-#include <pan/usenet-utils/gnksa.h>
+#include <pan/usenet-utils/filter-info.h>
 #include <pan/data/article.h>
 #include <pan/data/data.h>
-#include <pan/data/filter-info.h>
 #include <pan/icons/pan-pixbufs.h>
 #include "header-pane.h"
-#include "pad.h"
-#include "prefs-ui.h"
+#include "render-bytes.h"
+#include "sexy-icon-entry.h"
 #include "tango-colors.h"
 
 using namespace pan;
@@ -48,24 +45,13 @@ using namespace pan;
 *****
 ****/
 
-
 namespace
 {
   enum
   {
-    COL_DATE_STR,
-    COL_STATE,
-    COL_ACTION,
-    COL_SCORE,
-    COL_LINES,
-    COL_BYTES,
-    COL_DATE,
-    COL_ARTICLE_POINTER,
-    COL_COLLATED_SUBJECT,
-    COL_COLLATED_AUTHOR,
-    COL_SUBJECT,
-    COL_SHORT_AUTHOR,
-    N_COLUMNS
+    COL_DATE_STR, COL_STATE, COL_ACTION, COL_SCORE,
+    COL_LINES, COL_BYTES, COL_DATE, COL_ARTICLE_POINTER,
+    COL_SUBJECT, COL_SHORT_AUTHOR, N_COLUMNS
   };
 }
 
@@ -85,12 +71,6 @@ namespace
 {
   typedef std::set<const Article*> articles_t;
 
-  struct Icon
-  {
-    const guint8 * pixbuf_txt;
-    GdkPixbuf * pixbuf;
-  };
-
   enum
   {
     ICON_READ,
@@ -106,19 +86,21 @@ namespace
     ICON_QTY
    };
 
-   Icon _icons[ICON_QTY] =
-   {
-      { icon_article_read,           0 },
-      { icon_article_unread,         0 },
-      { icon_binary_complete,        0 },
-      { icon_binary_complete_read,   0 },
-      { icon_binary_incomplete,      0 },
-      { icon_binary_incomplete_read, 0 },
-      { icon_disk,                   0 },
-      { icon_bluecheck,              0 },
-      { icon_x,                      0 },
-      { icon_empty,                  0 }
-   };
+  struct Icon {
+    const guint8 * pixbuf_txt;
+    GdkPixbuf * pixbuf;
+  } _icons[ICON_QTY] = {
+    { icon_article_read,           0 },
+    { icon_article_unread,         0 },
+    { icon_binary_complete,        0 },
+    { icon_binary_complete_read,   0 },
+    { icon_binary_incomplete,      0 },
+    { icon_binary_incomplete_read, 0 },
+    { icon_disk,                   0 },
+    { icon_bluecheck,              0 },
+    { icon_x,                      0 },
+    { icon_empty,                  0 }
+  };
 
   const int
   get_article_state (const Data& data, const Article * a)
@@ -139,7 +121,7 @@ namespace
       retval = ICON_READ;
     else
       retval = ICON_UNREAD;
-                                                                                                                                              
+
     return retval;
   }
 
@@ -242,16 +224,11 @@ HeaderPane :: render_score (GtkTreeViewColumn * col,
   }
 
   g_object_set (renderer, "text", buf,
-  //                        "weight", PANGO_WEIGHT_NORMAL,
                           "background", (bg.empty() ? NULL : bg.c_str()),
                           "foreground", (fg.empty() ? NULL : fg.c_str()),
                           NULL);
 }
 
-#define KIBI 1024
-#define MEBI 1048576
-#define GIBI 1073741824
-
 void
 HeaderPane :: render_bytes (GtkTreeViewColumn * col,
                             GtkCellRenderer   * renderer,
@@ -261,18 +238,7 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * col,
 {
   unsigned long bytes (0);
   gtk_tree_model_get (model, iter, COL_BYTES, &bytes, -1);
-
-  char buf[128];
-  if (bytes < KIBI)
-    g_snprintf (buf, sizeof(buf), _("%lu B"), bytes);
-  else if (bytes < MEBI)
-    g_snprintf (buf, sizeof(buf), _("%.1f KiB"), (double)bytes/KIBI);
-  else if (bytes < GIBI)
-    g_snprintf (buf, sizeof(buf), _("%.1f MiB"), (double)bytes/MEBI);
-  else
-    g_snprintf (buf, sizeof(buf), _("%.1f GiB"), (double)bytes/GIBI);
-
-  g_object_set (renderer, "text", buf, NULL);
+  g_object_set (renderer, "text", pan::render_bytes(bytes), NULL);
 }
 
 struct HeaderPane::CountUnread: public PanTreeStore::WalkFunctor
@@ -335,11 +301,6 @@ HeaderPane :: render_subject (GtkTreeViewColumn * col,
     NULL);
 }
 
-namespace
-{
-  typedef std::vector<const Article*> article_v;
-}
-
 HeaderPane::Row*
 HeaderPane :: get_row (const Quark& message_id)
 {
@@ -362,6 +323,11 @@ HeaderPane :: create_row (const EvolutionDateMaker & e,
   return row;
 }
 
+namespace
+{
+  typedef std::vector<const Article*> article_v;
+}
+
 void
 HeaderPane ::  add_children_to_model (PanTreeStore               * store,
                                       PanTreeStore::Row          * parent_row,
@@ -482,8 +448,6 @@ HeaderPane :: build_model (const Quark               & group,
     G_TYPE_ULONG,   // bytes
     G_TYPE_ULONG,   //  date
     G_TYPE_POINTER, // article pointer
-    G_TYPE_STRING,  // collated subject (lazy)
-    G_TYPE_STRING,  // collated author (lazy)
     G_TYPE_STRING,  // subject
     G_TYPE_STRING); // short author
 
@@ -517,11 +481,23 @@ namespace
 void
 HeaderPane :: rebuild ()
 {
+  quarks_t selectme;
+  if (1) {
+    const articles_t old_selection (get_full_selection ());
+    foreach_const (articles_t, old_selection, it)
+      selectme.insert ((*it)->message_id);
+  }
+
   _mid_to_row.clear ();
   _tree_store = build_model (_group, _atree, NULL);
+
+  const bool sort_ascending = _prefs.get_flag ("header-pane-sort-ascending", false);
+  int sort_column = _prefs.get_int ("header-pane-sort-column", COL_DATE);
+  if (sort_column<0 || sort_column>=N_COLUMNS) // safeguard against odd settings
+    sort_column = COL_DATE;
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(_tree_store),
-                                        _prefs.get_int ("header-pane-sort-column", COL_DATE),
-                                        (_prefs.get_flag ("header-pane-sort-ascending", false) ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING));
+                                        sort_column,
+                                        sort_ascending ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
 
   GtkTreeModel * model (GTK_TREE_MODEL (_tree_store));
   GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
@@ -530,6 +506,9 @@ HeaderPane :: rebuild ()
 
   if (_prefs.get_flag ("expand-threads-when-entering-group", false))
     gtk_tree_view_expand_all (view);
+
+  if (!selectme.empty())
+    select_message_id (*selectme.begin());
 }
 
 bool
@@ -543,6 +522,9 @@ HeaderPane :: set_group (const Quark& new_group)
     if (!_group.empty() && _prefs.get_flag ("mark-group-read-when-leaving-group", false))
       _data.mark_group_read (_group);
 
+    if (_atree)
+        _atree->remove_listener (this);
+
     if (_tree_store)
     {
       save_sort_order (_prefs, _tree_store);
@@ -553,12 +535,8 @@ HeaderPane :: set_group (const Quark& new_group)
 
     _group = new_group;
 
-    if (_atree)
-    {
-      _atree->remove_listener (this);
-      delete _atree;
-      _atree = 0;
-    }
+    delete _atree;
+    _atree = 0;
 
     if (!_group.empty())
     {
@@ -567,7 +545,6 @@ HeaderPane :: set_group (const Quark& new_group)
 
       rebuild ();
 
-
       if (GTK_WIDGET_REALIZED(_tree_view))
         gtk_tree_view_scroll_to_point (GTK_TREE_VIEW(_tree_view), 0, 0);
     }
@@ -627,44 +604,32 @@ namespace
     }
   };
 
-  struct RememberArticles: public RowActionFunctor
+  struct RememberMessageId: public RowActionFunctor
   {
-    articles_t& articles;
-    RememberArticles (articles_t& a): articles(a) {}
-    virtual ~RememberArticles() {}
+    quarks_t& mids;
+    RememberMessageId (quarks_t& m): mids(m) {}
+    virtual ~RememberMessageId() {}
     virtual void operator() (GtkTreeModel* model, GtkTreeIter* it, const Article& article) {
-      articles.insert (&article);
+      mids.insert (article.message_id);
     }
   };
 }
 
-struct HeaderPane::SelectFirstArticle: public PanTreeStore::WalkFunctor
+void
+HeaderPane :: select_message_id (const Quark& mid, bool do_scroll)
 {
-  GtkTreeView * tree_view;
-  GtkTreeSelection * tree_selection;
-  const articles_t& articles;
-  const bool do_scroll;
-  SelectFirstArticle (GtkTreeView * v, GtkTreeSelection *sel, const articles_t& a, bool scroll):
-    tree_view(v), tree_selection(sel), articles(a), do_scroll(scroll) {}
-  virtual ~SelectFirstArticle () {}
-  virtual bool operator()(PanTreeStore *store, PanTreeStore::Row* r, GtkTreeIter *iter, GtkTreePath *unused) {
-    Row * row (dynamic_cast<Row*>(r));
-    const Article * article (row->article);
-    if (articles.count (article)) {
-      GtkTreePath * path = gtk_tree_model_get_path (GTK_TREE_MODEL(store), iter);
-      gtk_tree_view_expand_row (tree_view, path, true);
-      gtk_tree_view_expand_to_path (tree_view, path);
-      if (do_scroll) {
-        gtk_tree_view_set_cursor (tree_view, path, NULL, false);
-        gtk_tree_view_scroll_to_cell (tree_view, path, NULL, true, 0.5f, 0.0f);
-        gtk_tree_selection_select_path (tree_selection, path);
-      }
-      gtk_tree_path_free (path);
-      return false;
-    }
-    return true;
+  HeaderPane::Row * row = get_row (mid);
+  GtkTreePath * path (_tree_store->get_path(row));
+  GtkTreeView * view (GTK_TREE_VIEW(_tree_view));
+  gtk_tree_view_expand_to_path (view, path);
+  GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
+  gtk_tree_selection_select_path (sel, path);
+  if (do_scroll) {
+    gtk_tree_view_set_cursor (view, path, NULL, false);
+    gtk_tree_view_scroll_to_cell (view, path, NULL, true, 0.5f, 0.0f);
   }
-};
+  gtk_tree_path_free (path);
+}
 
 void
 HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
@@ -673,28 +638,37 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
       << diffs.reparented.size() << " reparented; "
       << diffs.removed.size() << " removed");
 
-  const bool rows_added_or_removed (!diffs.added.empty()
-                                 || !diffs.reparented.empty()
-                                 || !diffs.removed.empty());
-  articles_t selectme;
-  if (rows_added_or_removed)
-  {
-    // we might need to change the selection after the update.
-    const articles_t tmp (get_full_selection ());
+  // we might need to change the selection after the update.
+  const article_v old_selection (get_full_selection_v ());
+  quarks_t new_selection;
+  foreach_const (article_v, old_selection, it)
+    if (!diffs.removed.count ((*it)->message_id))
+        new_selection.insert ((*it)->message_id);
 
-    articles_t survivors;
-    foreach_const (articles_t, tmp, it)
-      if (!diffs.removed.count ((*it)->message_id))
-        survivors.insert (*it);
+  // if the old selection survived,
+  // is it visible on the screen?
+  bool selection_was_visible (true);
+#if GTK_CHECK_VERSION(2,8,0)
+  if (!new_selection.empty()) {
+    GtkTreeView *view (GTK_TREE_VIEW(_tree_view));
+    Row * row (get_row (*new_selection.begin()));
+    GtkTreePath *a(0), *b(0), *p (_tree_store->get_path (row));
+    gtk_tree_view_get_visible_range (view, &a, &b);
+    selection_was_visible = (gtk_tree_path_compare(a,p)<=0 &&
+                             gtk_tree_path_compare(p,b)<=0);
+    gtk_tree_path_free (a);
+    gtk_tree_path_free (b);
+    gtk_tree_path_free (p);
+  }
+#endif
 
-    if (survivors.empty()) {
-      // none of the current selections survive,
-      // so select the first article after the
-      // deleted ones...
-      ArticleIsNotInSet tester (diffs.removed);
-      RememberArticles actor (selectme);
-      action_next_if (tester, actor);
-    }
+  // if none of the current selection survived,
+  // we need to select something to replace the
+  // current selection.
+  if (!old_selection.empty() && new_selection.empty()) {
+    ArticleIsNotInSet tester (diffs.removed);
+    RememberMessageId actor (new_selection);
+    action_next_if (tester, actor);
   }
 
   // added...
@@ -703,10 +677,10 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     const EvolutionDateMaker date_maker;
     PanTreeStore::parent_to_children_t tmp;
     foreach_const (Data::ArticleTree::Diffs::added_t, diffs.added, it)
-      create_row (date_maker, _atree->get_article(it->message_id));
+      create_row (date_maker, _atree->get_article(it->first));
     foreach_const (Data::ArticleTree::Diffs::added_t, diffs.added, it) {
-      Row * parent (do_thread ? get_row (it->parent) : 0);
-      Row * child (get_row (it->message_id));
+      Row * parent (do_thread ? get_row (it->second.parent) : 0);
+      Row * child (get_row (it->first));
       tmp[parent].push_back (child);
     }
     _tree_store->insert_sorted (tmp);
@@ -716,8 +690,8 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   if (do_thread && !diffs.reparented.empty()) {
     PanTreeStore::parent_to_children_t tmp;
     foreach_const (Data::ArticleTree::Diffs::reparented_t, diffs.reparented, it) {
-      Row * parent (get_row (it->new_parent));
-      Row * child  (get_row (it->message_id));
+      Row * parent (get_row (it->second.new_parent));
+      Row * child  (get_row (it->second.message_id));
       tmp[parent].push_back (child);
     }
     _tree_store->reparent (tmp);
@@ -746,16 +720,14 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   if (!diffs.added.empty() && _prefs.get_flag ("expand-threads-when-entering-group", false))
     gtk_tree_view_expand_all (GTK_TREE_VIEW(_tree_view));
 
-  // select the next article if one was deleted...
-  // when an xover task adds a batch of articles periodically,
-  // scrolling causes an irritating 'jump', but when you delete
-  // articles, you /want/ that jump to the next article.
-  // So, scroll in the latter case but not the former.
-  if (!selectme.empty()) {
-    const bool do_scroll (diffs.added.empty() && !diffs.removed.empty());
-    GtkTreeSelection * sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(_tree_view));
-    SelectFirstArticle walk (GTK_TREE_VIEW(_tree_view), sel, selectme, do_scroll);
-    _tree_store->walk (walk);
+  // update our selection if necessary.
+  // if the new selection has just been added or reparented,
+  // and it was visible on the screen before,
+  // then scroll to ensure it's still visible.
+  if (!new_selection.empty()) {
+    const bool do_scroll = selection_was_visible
+            && (!diffs.added.empty() || !diffs.reparented.empty() || !diffs.removed.empty());
+    select_message_id (*new_selection.begin(), do_scroll);
   }
 }
 
@@ -965,9 +937,10 @@ HeaderPane :: on_button_pressed (GtkWidget * treeview, GdkEventButton *event, gp
   else if (self->_prefs.get_flag("single-click-activates-article",true)
            && (event->type == GDK_BUTTON_RELEASE)
            && (event->button == 1)
-           && (event->window == gtk_tree_view_get_bin_window (tv))
            && (event->send_event == false)
-           && !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK)))
+           && (event->window == gtk_tree_view_get_bin_window (tv))
+           && !(event->state & (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK))
+           && (self->get_full_selection_v().size() == 1u))
   {
     GtkTreePath * path;
     GtkTreeViewColumn * col;
@@ -1117,12 +1090,16 @@ HeaderPane :: rebuild_filter (const std::string& text, int mode)
   typedef std::pair<int,int> range_t;
   std::vector<range_t> ranges;
   ranges.reserve (6);
-  if (_action_manager.is_action_active("match-ignored-articles")) ranges.push_back (range_t(INT_MIN, -9999));
-  if (_action_manager.is_action_active("match-low-scoring-articles")) ranges.push_back (range_t(-9998, -1));
-  if (_action_manager.is_action_active("match-normal-scoring-articles")) ranges.push_back (range_t(0, 0));
-  if (_action_manager.is_action_active("match-medium-scoring-articles")) ranges.push_back (range_t(1, 4999));
-  if (_action_manager.is_action_active("match-high-scoring-articles")) ranges.push_back (range_t(5000, 9998));
-  if (_action_manager.is_action_active("match-watched-articles")) ranges.push_back (range_t(9999, INT_MAX));
+  if (_action_manager.is_action_active("match-only-watched-articles"))
+    ranges.push_back (range_t(9999, INT_MAX));
+  else {
+    if (_action_manager.is_action_active("match-ignored-articles")) ranges.push_back (range_t(INT_MIN, -9999));
+    if (_action_manager.is_action_active("match-low-scoring-articles")) ranges.push_back (range_t(-9998, -1));
+    if (_action_manager.is_action_active("match-normal-scoring-articles")) ranges.push_back (range_t(0, 0));
+    if (_action_manager.is_action_active("match-medium-scoring-articles")) ranges.push_back (range_t(1, 4999));
+    if (_action_manager.is_action_active("match-high-scoring-articles")) ranges.push_back (range_t(5000, 9998));
+    if (_action_manager.is_action_active("match-watched-articles")) ranges.push_back (range_t(9999, INT_MAX));
+  }
   for (size_t i=0; !ranges.empty() && i<ranges.size()-1; ) {
     if (ranges[i].second+1 != ranges[i+1].first)
       ++i;
@@ -1178,11 +1155,16 @@ HeaderPane :: filter (const std::string& text, int mode)
 {
   rebuild_filter (text, mode);
 
-  if (_atree) {
+  if (_atree)
+  {
+    _wait.watch_cursor_on ();
+
     if (_filter._aggregates.empty())
       _atree->set_filter ();
     else
       _atree->set_filter (_show_type, &_filter);
+
+    _wait.watch_cursor_off ();
   }
 }
 
@@ -1253,14 +1235,6 @@ namespace
     remove_activate_soon_tag ();
   }
 
-  void reset_entry_filter_cb (GtkWidget * button, gpointer pane_gpointer)
-  {
-    GtkWidget * e = GTK_WIDGET (g_object_get_data (G_OBJECT(button), "entry"));
-    set_search_entry (e, "");
-    search_text.clear ();
-    search_entry_activated (NULL, pane_gpointer);
-  }
-
   gboolean activated_timeout_cb (gpointer h_gpointer)
   {
     search_activate (static_cast<HeaderPane*>(h_gpointer));
@@ -1269,22 +1243,20 @@ namespace
   }
 
   // ensure there's exactly one activation timeout
-  // and that it's set to go off one second from now.
+  // and that it's set to go off in a half second from now.
   void bump_activate_soon_tag (HeaderPane * h)
   {
     remove_activate_soon_tag ();
-    activate_soon_tag = g_timeout_add (1000, activated_timeout_cb, h);
+    activate_soon_tag = g_timeout_add (500, activated_timeout_cb, h);
   }
 
   // when the user changes the filter text,
   // update our state variable and bump the activate timeout.
-  void search_entry_changed_by_user (GtkEditable * e, gpointer h_gpointer)
+  void search_entry_changed (GtkEditable * e, gpointer h_gpointer)
   {
     search_text = gtk_entry_get_text (GTK_ENTRY(e));
     bump_activate_soon_tag (static_cast<HeaderPane*>(h_gpointer));
-
-    GtkWidget * b (GTK_WIDGET (g_object_get_data (G_OBJECT(e), "reset-button")));
-    gtk_widget_set_sensitive (b, !search_text.empty());
+    refresh_search_entry (GTK_WIDGET(e));
   }
 
   // when the search mode is changed via the menu,
@@ -1302,15 +1274,23 @@ namespace
     }
   }
 
-  // this pops up the `author, subject, subject-or-author' menu
-  // when the search mode button is clicked.
-  void search_mode_button_clicked_cb (GtkButton * button, gpointer menu_g)
+  void entry_icon_released (SexyIconEntry *entry, SexyIconEntryPosition icon_pos, int button, gpointer menu)
   {
-    gtk_menu_popup (GTK_MENU(menu_g), 0, 0,
-                    0, 0,
-                    0, gtk_get_current_event_time());
+    if (icon_pos == SEXY_ICON_ENTRY_PRIMARY)
+      gtk_menu_popup (GTK_MENU(menu), 0, 0, 0, 0, 0, gtk_get_current_event_time());
   }
 
+  void entry_icon_released_2 (SexyIconEntry *entry, SexyIconEntryPosition icon_pos, int button, gpointer pane_gpointer)
+  {
+    if (icon_pos == SEXY_ICON_ENTRY_SECONDARY) {
+      set_search_entry (GTK_WIDGET(entry), "");
+      refresh_search_entry (GTK_WIDGET(entry));
+      search_text.clear ();
+      search_entry_activated (NULL, pane_gpointer);
+    }
+  }
+
+
   void ellipsize_if_supported (GObject * o)
   {
 #if GTK_CHECK_VERSION(2,6,0)
@@ -1372,8 +1352,8 @@ HeaderPane :: build_tree_columns ()
     else if (name == "subject")
     {
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
-        "ypad", 0,
         "xpad", xpad,
+        "ypad", 0,
         NULL));
       ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, NULL);
@@ -1388,8 +1368,8 @@ HeaderPane :: build_tree_columns ()
     else if (name == "score")
     {
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
-        "ypad", 0,
         "xpad", xpad,
+        "ypad", 0,
         "xalign", 1.0,
         NULL));
       ellipsize_if_supported (G_OBJECT(r));
@@ -1507,14 +1487,21 @@ HeaderPane :: ~HeaderPane ()
 GtkWidget*
 HeaderPane :: create_filter_entry ()
 {
-  GtkWidget * entry = gtk_entry_new ();
+  GtkWidget * entry = sexy_icon_entry_new ();
   _action_manager.disable_accelerators_when_focused (entry);
   g_object_set_data (G_OBJECT(entry), "header-pane", this);
   g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
   g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), NULL);
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
-  entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed_by_user), this);
-  refresh_search_entry (entry);
+  entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed), this);
+
+  GtkWidget * image = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
+  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, GTK_IMAGE(image));
+  sexy_icon_entry_set_icon_highlight(SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_SECONDARY, true);
+
+  image = gtk_image_new_from_stock ("ICON_SEARCH_PULLDOWN", GTK_ICON_SIZE_MENU);
+  sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_PRIMARY, GTK_IMAGE(image));
+  sexy_icon_entry_set_icon_highlight (SEXY_ICON_ENTRY(entry), SEXY_ICON_ENTRY_PRIMARY, TRUE);
 
   GtkWidget * menu = gtk_menu_new ();
   mode = 0;
@@ -1527,34 +1514,12 @@ HeaderPane :: create_filter_entry ()
     gtk_menu_shell_append (GTK_MENU_SHELL(menu), w);
     gtk_widget_show (w);
   }
+  g_signal_connect (entry, "icon-released", G_CALLBACK(entry_icon_released), menu);
+  g_signal_connect (entry, "icon-released", G_CALLBACK(entry_icon_released_2), this);
 
-  GtkWidget * search_mode_button = gtk_button_new ();
-  g_signal_connect (search_mode_button, "clicked", G_CALLBACK(search_mode_button_clicked_cb), menu);
-  GtkWidget * image = gtk_image_new_from_stock ("ICON_SEARCH_PULLDOWN", GTK_ICON_SIZE_BUTTON);
-  gtk_container_add (GTK_CONTAINER(search_mode_button), image);
-  gtk_button_set_relief (GTK_BUTTON(search_mode_button), GTK_RELIEF_NONE);
+  refresh_search_entry (entry);
 
-  GtkTooltips * tips = gtk_tooltips_new ();
-  GtkWidget * w = gtk_button_new ();
-  g_object_set_data (G_OBJECT(w), "entry", entry);
-  g_object_set_data (G_OBJECT(entry), "reset-button", w);
-  gtk_button_set_relief (GTK_BUTTON(w), GTK_RELIEF_NONE);
-  g_signal_connect (w, "clicked", G_CALLBACK(reset_entry_filter_cb), this);
-  image = gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON);
-  gtk_container_add (GTK_CONTAINER(w), image);
-  gtk_tooltips_set_tip (GTK_TOOLTIPS(tips), w, _("Clear the Filter"), NULL);
-  gtk_widget_set_sensitive (w, false);
-
-  // when the button's destroyed, free the tips too
-  g_object_ref (tips);
-  gtk_object_sink (GTK_OBJECT (tips));
-  g_signal_connect_swapped (w, "destroy", G_CALLBACK(g_object_unref), tips);
-
-  GtkWidget * box = gtk_hbox_new (false, 0);
-  gtk_box_pack_start (GTK_BOX(box), search_mode_button, false, false, 0);
-  gtk_box_pack_start (GTK_BOX(box), entry, true, true, 0);
-  gtk_box_pack_start (GTK_BOX(box), w, false, false, 0);
-  return box;
+  return entry;
 }
 
 Data::ShowType _show_type;
@@ -1563,11 +1528,13 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
                           Data                & data,
                           Queue               & queue,
                           ArticleCache        & cache,
-                          Prefs               & prefs):
+                          Prefs               & prefs,
+                          WaitUI              & wait):
   _action_manager (action_manager),
   _data (data),
   _queue (queue),
   _prefs (prefs),
+  _wait (wait),
   _atree (0),
   _root (0),
   _tree_view (0),
@@ -1726,120 +1693,24 @@ namespace
   ***  Navigation Functors
   **/
 
-  struct TreeIteratorNext: public TreeIterFunctor
-  {
+  struct TreeIteratorNext: public TreeIterFunctor {
     virtual ~TreeIteratorNext () {}
-
-    virtual bool operator ()(GtkTreeModel * model, GtkTreeIter * iter) const
-    {
-      g_assert (model != 0);
-      g_assert (iter != 0);
-
-      // childen
-      GtkTreeIter tmp;
-      if (gtk_tree_model_iter_nth_child (model, &tmp, iter, 0)) {
-        *iter = tmp;
-        return true;
-      }
-
-      // siblings
-      GtkTreeIter up = *iter;
-      do {
-        tmp = up;
-        if (gtk_tree_model_iter_next (model, &up)) {
-          *iter = up;
-          return true;
-        }
-      } while (gtk_tree_model_iter_parent (model, &up, &tmp));
-
-      // if all else fails, return the first node
-      if (gtk_tree_model_get_iter_first (model, iter))
-        return true;
-
-      return false;
+    virtual bool operator ()(GtkTreeModel * model, GtkTreeIter * iter) const {
+      return model && PAN_TREE_STORE(model)->get_next (iter);
     }
-
     virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
-      const bool ok = gtk_tree_model_get_iter_first (model, setme);
-      return ok;
+      return model && PAN_TREE_STORE(model)->front (setme);
     }
   };
 
-  struct TreeIteratorPrev: public TreeIterFunctor
-  {
-    private:
-
-      // only works for iterators of a tree store.
-      // treestore's iter->user_data is a GNode pointer...
-      bool
-      tree_store_iter_equal (const GtkTreeIter * a, const GtkTreeIter * b) const
-      {
-        if (!a && !b) return true;
-        if (!a || !b) return false;
-        return a->user_data == b->user_data;
-      }
-
-      void
-      tree_model_iter_last_descendant (GtkTreeModel * model,
-                                       GtkTreeIter * setme,
-                                       GtkTreeIter * parent) const
-      {
-        GtkTreeIter march (*parent);
-        for (;;) {
-          const int n_children (gtk_tree_model_iter_n_children (model, &march));
-          if (!n_children) {
-            *setme = march;
-            return;
-          }
-          GtkTreeIter tmp;
-          gtk_tree_model_iter_nth_child (model, &tmp, &march, n_children-1);
-          march = tmp;
-        }
-      }
-
-    public:
-
-      virtual ~TreeIteratorPrev () {}
-
-      bool
-      operator()(GtkTreeModel * model, GtkTreeIter * iter) const
-      {
-        // get parent
-        GtkTreeIter parent;
-        const bool has_parent = gtk_tree_model_iter_parent (model, &parent, iter);
-
-        // get sibling
-        GtkTreeIter sibling;
-        gtk_tree_model_iter_children (model, &sibling, (has_parent ? &parent : 0));
-        if (tree_store_iter_equal (&sibling, iter)) { // iter is the first child
-          *iter = parent;
-          return has_parent;
-        }
-
-        // walk along siblings until we find the one prior to iter.
-        // gtk_tree_model_iter_prev() would have been nice here.
-        for (;;) {
-          GtkTreeIter tmp = sibling;
-          g_assert (gtk_tree_model_iter_next (model, &tmp));
-          if (tree_store_iter_equal (&tmp, iter))
-            break;
-          sibling = tmp;
-        }
-
-        // find the absolutely last child of the older sibling
-        tree_model_iter_last_descendant (model, iter, &sibling);
-        return true;
-      }
-
-      // aka gtk_tree_model_get_iter_last (model, setme);
-      virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
-        const int n (gtk_tree_model_iter_n_children (model, NULL));
-        GtkTreeIter tmp;
-        if (!gtk_tree_model_iter_nth_child (model, &tmp, NULL, n-1))
-          return false;
-        tree_model_iter_last_descendant (model, setme, &tmp);
-        return true;
-      }
+  struct TreeIteratorPrev: public TreeIterFunctor {
+    virtual ~TreeIteratorPrev () {}
+    bool operator()(GtkTreeModel * model, GtkTreeIter * iter) const {
+      return model && PAN_TREE_STORE(model)->get_prev (iter);
+    }
+    virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
+      return model && PAN_TREE_STORE(model)->back (setme);
+    }
   };
 
   /**
