@@ -57,7 +57,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uuscan_id = "$Id: uuscan.c,v 1.1.1.1 2006/10/02 20:45:57 csk Exp $";
+char * uuscan_id = "$Id: uuscan.c 260 2007-05-04 23:32:54Z csk $";
 
 /*
  * Header fields we recognize as such. See RFC822. We add "From ",
@@ -170,7 +170,7 @@ ScanHeaderLine (FILE *datei, char *initial)
 {
   char *ptr=uuscan_shlline;
   char *ptr2, *p1, *p2, *p3;
-  int llength, c;
+  int llength;
   long curpos;
   int hadcr;
 
@@ -202,7 +202,7 @@ ScanHeaderLine (FILE *datei, char *initial)
   }
 
   while (!feof (datei)) {
-    c = fgetc (datei);
+    char c = fgetc (datei);
     if (feof (datei))
       break;
 
@@ -309,6 +309,7 @@ ScanHeaderLine (FILE *datei, char *initial)
 	}
 
 	if (*p3 == '?' && *(p3+1) == '=') {
+          char c = -1;
 	  /*
 	   * Alright, we've found an RFC 1522 header field
 	   */
@@ -573,7 +574,7 @@ IsKnownHeader (char *line)
     }
   }
 
-  return 0;
+  return retval;
 }
 
 /*
